Package: DyadRatios
Type: Package
Title: Dyad Ratios Algorithm for Latent Variable Estimation
Version: 2.1
Date: 2026-05-11
Authors@R: c(
    person("James", "Stimson", role = c("aut"), comment = "Original C++ implementation"),
    person("Dave", "Armstrong", role = c("cre", "aut"), email = "davearmstrong.ps@gmail.com")
    )
Description: Implements the Dyad Ratios algorithm for estimating latent
    variables from time-series survey data. The algorithm estimates a
    latent mood dimension (or two dimensions) from a set of issue opinion
    series. Supports annual, quarterly, monthly, and daily aggregation
    intervals, optional exponential smoothing, and up to two latent
    dimensions. Input data can be provided as a data frame or read from
    delimited text files. Based on Stimson's 'MCalc' C++ program. 
    See Stimson (2018) <doi:10.1177/0759106318761614> for more details. 
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: Rcpp (>= 1.0.0)
LinkingTo: Rcpp
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Suggests: testthat (>= 3.0.0), ggplot2, knitr, rmarkdown, dplyr,
        lubridate, rio, tidyr
NeedsCompilation: yes
Packaged: 2026-05-11 17:31:14 UTC; david
Author: James Stimson [aut] (Original C++ implementation),
  Dave Armstrong [cre, aut]
Maintainer: Dave Armstrong <davearmstrong.ps@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 19:00:14 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-05-11 21:07:43 UTC; unix
Archs: DyadRatios.so.dSYM

Package: DyadRatios
Type: Package
Title: Dyad Ratios Algorithm for Latent Variable Estimation
Version: 2.1
Date: 2026-05-11
Authors@R: c(
    person("James", "Stimson", role = c("aut"), comment = "Original C++ implementation"),
    person("Dave", "Armstrong", role = c("cre", "aut"), email = "davearmstrong.ps@gmail.com")
    )
Description: Implements the Dyad Ratios algorithm for estimating latent
    variables from time-series survey data. The algorithm estimates a
    latent mood dimension (or two dimensions) from a set of issue opinion
    series. Supports annual, quarterly, monthly, and daily aggregation
    intervals, optional exponential smoothing, and up to two latent
    dimensions. Input data can be provided as a data frame or read from
    delimited text files. Based on Stimson's 'MCalc' C++ program. 
    See Stimson (2018) <doi:10.1177/0759106318761614> for more details. 
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: Rcpp (>= 1.0.0)
LinkingTo: Rcpp
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Suggests: testthat (>= 3.0.0), ggplot2, knitr, rmarkdown, dplyr,
        lubridate, rio, tidyr
NeedsCompilation: yes
Packaged: 2026-05-11 17:31:14 UTC; david
Author: James Stimson [aut] (Original C++ implementation),
  Dave Armstrong [cre, aut]
Maintainer: Dave Armstrong <davearmstrong.ps@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 19:00:14 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-11 19:27:17 UTC; unix
Archs: DyadRatios.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,boot_dr)
S3method(plot,extract)
S3method(print,extract)
S3method(summary,extract)
export(boot_dr)
export(extract)
export(get_mood)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,adjustcolor)
importFrom(graphics,abline)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,sd)
importFrom(utils,combn)
useDynLib(DyadRatios, .registration = TRUE)

#' Run MCalc non-interactively
#'
#' Constructs the sequence of keystrokes that MCalc expects, writes them to a
#' temporary file, and pipes that file into the MCalc executable via
#' \code{system()}.  The first argument can be either a path to an existing
#' MCalc-format data file \emph{or} an R data frame; in the latter case the
#' data are written to a temporary file in the correct format before being
#' passed to MCalc.
#'
#' @param data Either a character string giving the full path to a
#'   MCalc-format data file, \emph{or} a data frame containing the survey
#'   series.  When a data frame is supplied the columns specified by
#'   \code{varname_col}, \code{date_col}, \code{index_col}, and \code{n_col}
#'   are used to build the input file automatically.
#' @param varname_col Character. Name of the column in \code{data} that holds
#'   the variable/series identifier.  Default \code{"varname"}.  Ignored when
#'   \code{data} is a file path.
#' @param date_col Character. Name of the column in \code{data} that holds the
#'   observation date (a \code{Date} object or a character string in any
#'   unambiguous format recognised by \code{\link[base]{as.Date}}).
#'   Default \code{"date"}.  Ignored when \code{data} is a file path.
#' @param index_col Character. Name of the column that holds the numeric index
#'   value (e.g. percentage agreeing).  Default \code{"index"}.  Ignored when
#'   \code{data} is a file path.
#' @param n_col Character or \code{NULL}. Name of the column that holds the
#'   sample size for each observation.  When \code{NULL} (default) a
#'   placeholder of \code{1000} is used for every row, which leaves relative
#'   weighting uniform.  Ignored when \code{data} is a file path.
#' @param agg_interval Character. Aggregation interval: \code{"a"} (annual,
#'   default), \code{"q"} (quarterly), \code{"m"} (monthly), \code{"d"}
#'   (daily), or \code{"o"} (multiple years).
#' @param n_dim Integer. Number of dimensions to extract: \code{1} (default)
#'   or \code{2}.
#' @param smoothing Logical. \code{TRUE} (default) turns exponential smoothing
#'   on; \code{FALSE} turns it off.
#' @param start_year Integer or \code{NULL}. Restrict estimation to periods
#'   on or after this year.  \code{NULL} (default) accepts MCalc's earliest
#'   available date.
#' @param start_period Integer or \code{NULL}. For quarterly/monthly/daily
#'   aggregation, the sub-annual start period (quarter 1–4, month 01–12, or
#'   day 01–31).  Ignored for annual aggregation.  \code{NULL} uses period 1.
#' @param end_year Integer or \code{NULL}. Restrict estimation to periods on
#'   or before this year.  \code{NULL} (default) accepts MCalc's latest
#'   available date.
#' @param end_period Integer or \code{NULL}. Sub-annual end period.
#'   \code{NULL} uses the last period of the year (4, 12, or 31).
#' @param output_root Character or \code{NULL}. Root name for the output
#'   \code{.log} and \code{.csv} files.  When \code{NULL} and \code{data} is a
#'   file path, MCalc uses the input file's stem.  When \code{NULL} and
#'   \code{data} is a data frame, defaults to \code{"mcalc_output"}.
#' @param mcalc_path Character. Full path to the MCalc executable.  Defaults
#'   to \code{"/usr/local/bin/MCalc64"}.
#' @param read_output Logical. If \code{TRUE} (default), reads and returns the
#'   \code{.csv} output as a data frame.  If \code{FALSE}, returns only the
#'   file paths and console output.
#'
#' @return A list with elements:
#'   \describe{
#'     \item{\code{console}}{Character vector of lines printed to stdout.}
#'     \item{\code{log_file}}{Path to the \code{.log} output file.}
#'     \item{\code{csv_file}}{Path to the \code{.csv} output file.}
#'     \item{\code{data}}{Data frame read from the \code{.csv} file (only
#'       present when \code{read_output = TRUE} and the file exists).}
#'   }
#'
#' @examples
#' \dontrun{
#' # --- from a file path ---------------------------------------------------
#' out <- run_mcalc("/Users/me/data/mysurvey.txt",
#'                  mcalc_path = "/Users/me/bin/MCalc64")
#' head(out$data)
#'
#' # --- from a data frame --------------------------------------------------
#' # with default column names (varname, date, index, n)
#' out2 <- run_mcalc(my_df,
#'                   n_dim      = 2,
#'                   smoothing  = FALSE,
#'                   mcalc_path = "/Users/me/bin/MCalc64")
#'
#' # with custom column names
#' out3 <- run_mcalc(survey_df,
#'                   varname_col = "item",
#'                   date_col    = "fielddate",
#'                   index_col   = "pct_agree",
#'                   n_col       = "sample_size",
#'                   start_year  = 1990,
#'                   end_year    = 2020,
#'                   mcalc_path  = "/Users/me/bin/MCalc64")
#' }
#'
#' @export
run_mcalc <- function(data,
                      varname_col  = "varname",
                      date_col     = "date",
                      index_col    = "index",
                      n_col        = NULL,
                      agg_interval = "a",
                      n_dim        = 1L,
                      smoothing    = TRUE,
                      start_year   = NULL,
                      start_period = NULL,
                      end_year     = NULL,
                      end_period   = NULL,
                      output_root  = NULL,
                      mcalc_path   = "/usr/local/bin/MCalc64",
                      read_output  = TRUE) {

  # --- validate shared arguments ----------------------------------------------
  agg_interval <- match.arg(agg_interval, c("a", "q", "m", "d", "o"))
  n_dim        <- as.integer(n_dim)
  if (!n_dim %in% 1:2) stop("n_dim must be 1 or 2")
  if (!file.exists(mcalc_path))
    stop("MCalc executable not found at: ", mcalc_path)

  # --- resolve input: file path or data frame ---------------------------------
  temp_data_file <- NULL   # will be set (and later cleaned up) for data frames

  if (is.data.frame(data)) {

    # -- validate column names -------------------------------------------------
    required_cols <- c(varname_col, date_col, index_col)
    missing_cols  <- setdiff(required_cols, names(data))
    if (length(missing_cols) > 0)
      stop("Column(s) not found in data: ",
           paste(missing_cols, collapse = ", "))
    if (!is.null(n_col) && !n_col %in% names(data))
      stop("n_col '", n_col, "' not found in data")

    # -- coerce and format columns ---------------------------------------------
    varnames <- as.character(data[[varname_col]])
    dates    <- as.Date(data[[date_col]])   # handles Date objects and strings
    if (anyNA(dates))
      stop("date_col contains values that could not be parsed as dates")
    date_str <- format(dates, "%m/%d/%Y")  # MCalc expects MM/DD/YYYY
    index    <- as.numeric(data[[index_col]])
    n_vals   <- if (is.null(n_col)) rep(1000L, nrow(data))
                else as.integer(data[[n_col]])

    # -- write to temp file (same dir so output lands somewhere predictable) ---
    temp_dir       <- tempdir()
    temp_data_file <- file.path(temp_dir, "mcalc_input.txt")
    lines          <- paste(varnames, date_str, index, n_vals)
    writeLines(lines, temp_data_file)

    input_file  <- temp_data_file
    output_dir  <- temp_dir
    # default root name for data-frame input (temp file stem would be ugly)
    if (is.null(output_root)) output_root <- "mcalc_output"

  } else if (is.character(data) && length(data) == 1) {

    input_file  <- normalizePath(data, mustWork = TRUE)
    output_dir  <- dirname(input_file)
    # output_root stays NULL → MCalc uses the input file stem

  } else {
    stop("'data' must be either a data frame or a single file path string")
  }

  # --- build the stdin sequence -----------------------------------------------
  inputs <- character(0)

  # MCalc saves the last-used filename in ~/LastFile.txt and prompts to reuse it
  last_file_path <- path.expand("~/LastFile.txt")
  if (file.exists(last_file_path)) {
    inputs <- c(inputs, "n")        # don't reuse — supply new path below
  }
  inputs <- c(inputs, input_file)   # full path to data file

  inputs <- c(inputs, agg_interval)
  if (agg_interval == "o") inputs <- c(inputs, "2")  # multiple-year factor

  inputs <- c(inputs, "n")   # topic-code selection

  # start date -----------------------------------------------------------------
  if (is.null(start_year)) {
    inputs <- c(inputs, "y")
  } else {
    inputs <- c(inputs, "n", as.character(as.integer(start_year)))
    if (agg_interval == "q") {
      inputs <- c(inputs,
                  as.character(if (is.null(start_period)) 1L
                               else as.integer(start_period)))
    } else if (agg_interval %in% c("m", "d")) {
      inputs <- c(inputs,
                  sprintf("%02d", if (is.null(start_period)) 1L
                                  else as.integer(start_period)))
    }
    if (agg_interval == "d") inputs <- c(inputs, "01")
  }

  # end date -------------------------------------------------------------------
  if (is.null(end_year)) {
    inputs <- c(inputs, "y")
  } else {
    inputs <- c(inputs, "n", as.character(as.integer(end_year)))
    if (agg_interval == "q") {
      inputs <- c(inputs,
                  as.character(if (is.null(end_period)) 4L
                               else as.integer(end_period)))
    } else if (agg_interval %in% c("m", "d")) {
      inputs <- c(inputs,
                  sprintf("%02d", if (is.null(end_period)) 12L
                                  else as.integer(end_period)))
    }
    if (agg_interval == "d") inputs <- c(inputs, "31")
  }

  inputs <- c(inputs, as.character(n_dim))
  inputs <- c(inputs, if (smoothing) "y" else "n")
  inputs <- c(inputs, if (is.null(output_root)) "y" else output_root)

  # --- write stdin sequence and run MCalc -------------------------------------
  stdin_file <- tempfile(fileext = ".txt")
  on.exit({
    unlink(stdin_file)
    if (!is.null(temp_data_file)) unlink(temp_data_file)
  }, add = TRUE)
  writeLines(inputs, stdin_file)

  cmd     <- paste(shQuote(mcalc_path), "<", shQuote(stdin_file))
  console <- system(cmd, intern = TRUE)

  # --- determine output file paths --------------------------------------------
  root     <- if (is.null(output_root))
                tools::file_path_sans_ext(basename(input_file))
              else output_root
  log_path <- file.path(output_dir, paste0(root, ".log"))
  csv_path <- file.path(output_dir, paste0(root, ".csv"))

  result <- list(
    console  = console,
    log_file = log_path,
    csv_file = csv_path
  )

  if (read_output && file.exists(csv_path)) {
    result$data <- utils::read.csv(csv_path, stringsAsFactors = FALSE)
  }

  result
}

#' Residualize issues for first mood dimension.  Low level function called by extract, not intended to be called by the user. 
#' @noRd
residmi <- function(issue,v,mood) { #function regresses issue(v) on mood and then residualizes it
o<- lm(issue[,v] ~ mood[3,]) #regress issue on mood to get a,b
issue[,v]<- 100 + issue[,v] - (o$coef[1]+o$coef[2]*mood[3,]) #100 + Y - (a+bx)
return(issue[,v])
}

globalVariables(c("period", "latent1", "latent2"))

#' Plot Method for the extract Function. 
#' 
#' This function generates a line plot of the latent variable estimates obtained from the `extract()` function. It can handle both one-dimensional and two-dimensional latent variable estimates.
#' 
#' @param x An object of class "extract", typically a result of the `extract()` function.
#' @param ... Additional graphical parameters to be passed to the plot function.
#' @return A ggplot of the latent variable estimate(s) over time. 
#' @usage NULL
#' @method plot extract
#' @importFrom ggplot2 ggplot geom_line labs aes
#' @export
#' @examples
#' data(jennings)
#' dr_out <- extract(varname = jennings$variable, 
#'                   date = jennings$date, 
#'                   index = jennings$value, 
#'                   ncases = jennings$n, 
#'                   begindt = min(jennings$date), 
#'                   enddt = max(jennings$date), 
#'                   npass=1)
#' plot(dr_out)
plot.extract <- function(x, ...) {
  mood <- get_mood(x)
  if(!("latent2" %in% names(x))){
    g <- ggplot(mood, aes(x=period, y=latent1)) + 
      geom_line() +
      labs(x="Time Period", y="Dimension 1 Estimate") 
  }else{
    g <- ggplot(mood, aes(x=period)) + 
      geom_line(aes(y=latent1, color="Dimension 1")) +
      geom_line(aes(y=latent2, color="Dimension 2")) +
      labs(x="Time Period", y="Latent Variable Estimates", color="Legend") 
  }
  return(g)
  }

#' Dyad Ratios Algorithm
#'
#' Estimates the Dyad Ratios Algorithm for constructing latent time series from
#' survey-research marginals.
#'
#' @param varname String giving the name of the input series to be smoothed.
#'   This should identify similar or comparable values in the series. Values in
#'   the series that have the same \code{varname} will be assumed to come from
#'   the same source.
#' @param date ISO numeric representation of the date the survey was in the
#'   field (usually the start, end, or median date).
#' @param index Numeric value of the series. It might be a percent or proportion
#'   responding in a single category (e.g., the approve response in presidential
#'   approval) or some multi-response summary. For ease of interpretation,
#'   polarity should be the same for all items.
#' @param ncases Number of cases (e.g., sample size) of the survey. This provides
#'   differential weighting for the values. Setting this to \code{NULL} or
#'   leaving it blank will weight each value equally.
#' @param unit Aggregation period—one of \sQuote{D} (daily), \sQuote{M}
#'   (monthly), \sQuote{Q} (quarterly), \sQuote{A} (annual), or \sQuote{O}
#'   (multi-year aggregation).
#' @param mult Number of years, only used if \code{unit} is \sQuote{O}.
#' @param begindt Beginning date of the analysis. Defaults to earliest date in
#'   the dataset. Should be specified with \code{lubridate::ymd()}.
#' @param enddt Ending date for the analysis. Defaults to the latest date in the
#'   data.
#' @param npass Not yet implemented.
#' @param smoothing Logical. Specifies whether exponential smoothing is applied
#'   to the intermediate estimates during the iterative solution process.
#'   Defaults to \code{TRUE}.
#' @param endmonth Ending month of the analysis.
#' @param R Number of bootstrap samples.
#' @param parallel Logical indicating whether the `mclapply` function should be used. 
#' @param level The confidence level for the intervals. Default is 0.95. 
#' @param pw Logical indicating whether to do pairwise tests. 
#' @param ... Other arguments to be passed down to `mclapply`.  
#'
#' @return A list with potentially two data frames `ci` has variables:
#' \itemize{
#'   \item \code{period}: Aggregation period. 
#'   \item \code{latent1}: Estimate of latent variable from original analysis.
#'   \item \code{lwr}: Lower confidence bound.
#'   \item \code{upr}: Upper confidence bound. 
#' }
#' If `pw = TRUE`, the list also contains `pw` with variables:
#' \itemize{
#'   \item \code{p1}: Earlier period 
#'   \item \code{p2}: Later period 
#'   \item \code{diff}: (mood for p2) - (mood for p1)
#'   \item \code{p_diff}: Probability that the larger mood is bigger than the smaller mood. 
#' }
#'
#' @references
#' Stimson, J. A. (2018).
#' \sQuote{The Dyad Ratios Algorithm for Estimating Latent Public Opinion:
#' Estimation, Testing, and Comparison to Other Approaches},
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie
#' Sociologique}, 137–138(1), 201–218. \doi{10.1177/0759106318761614}
#'
#' @importFrom stats cor lm optim sd var quantile aggregate rbinom reshape
#' @importFrom graphics legend lines par plot 
#' @importFrom lubridate ymd year month day quarter
#' @importFrom progress progress_bar
#' @importFrom utils combn
#' 
#' @usage NULL
#' @export
#'
#' @examples
#' data(jennings)
#' # R should be higher for real-world applications
#' \dontrun{
#' boot_out <- boot_dr(varname = jennings$variable, 
#'                   date = jennings$date, 
#'                   index = jennings$value, 
#'                   ncases = jennings$n, 
#'                   begindt = as.Date("1985-01-01"), 
#'                   enddt = max(jennings$date), 
#'                   npass=1, R=1000, 
#'                   parallel=FALSE)
#' boot_out
#' }
boot_dr <- function(varname,
                    date,
                    index,
                    ncases=NULL,
                    unit="A",
                    mult=1,
                    begindt=NA,
                    enddt=NA,
                    npass=1,
                    smoothing=TRUE,
                    endmonth=12, 
                    R=1000, 
                    parallel=FALSE, 
                    level = .95,
                    pw = FALSE, 
                    ...) { 
  if(is.null(ncases))ncases <- rep(1000, length(index))
  if(!is.null(ncases) & any(is.na(ncases)))ncases <- ifelse(is.na(ncases), 1000, ncases)
  if(any(index > 1))index <- index/100
  e_orig <- DyadRatios::extract(varname, date, index, ncases, unit, mult, begindt, enddt, npass, smoothing, endmonth)
  if(!parallel){
    pb <- progress_bar$new(total = R)
    moods <- vector(mode="list", length=R)
    for(iter in 1:R){
      n_cases_boot <- rbinom(n = length(index), prob= index, size=ncases)
      index_boot <- n_cases_boot/ncases
      e <- extract_bs(varname, date, index_boot, ncases, unit, mult, begindt, enddt, npass, smoothing, endmonth, wm=e_orig$wtdmean, ws=e_orig$wtdstd)
      out <- get_mood(e)
      out$iter <- iter
      moods[[iter]] <- out
      pb$tick()
    }
  }else{
    if(!requireNamespace("parallel", quietly = TRUE)){
      stop("Must install parallel package if parallel=TRUE.\n")
    }
    moods <- parallel::mclapply(1:R, \(i){
      n_cases_boot <- rbinom(n = length(index), prob= index, size=ncases)
      index_boot <- n_cases_boot/ncases
      e <- extract_bs(varname, date, index_boot, ncases, unit, mult, begindt, enddt, npass, smoothing, endmonth, wm=e_orig$wtdmean, ws=e_orig$wtdstd)
      out <- get_mood(e)
      out$iter <- i
      out
    }, ...)
    
  }
  lwr_level <- (1 - level)/2
  upr_level <- 1 - lwr_level
  moods <- as.data.frame(do.call(rbind, moods))
  if(pw){
    moodsw <- reshape(moods, idvar = "period", timevar = "iter", direction = "wide")
    pd <- moodsw$period
    moods_mat <- t(as.matrix(moodsw[,-1]))
    combs <- combn(ncol(moods_mat), 2)
    D <- matrix(0, nrow=ncol(moods_mat), ncol=ncol(combs))
    D[cbind(combs[1,], 1:ncol(combs))] <- -1
    D[cbind(combs[2,], 1:ncol(combs))] <- 1
    diffs <- moods_mat %*% D
    p_diff <- apply(diffs, 2, \(x)mean(x > 0))
    p_diff <- ifelse(p_diff < .5, 1-p_diff, p_diff)
    diff = matrix(get_mood(e_orig)$latent1, nrow=1) %*% D
    res_pw <- data.frame(p1 = pd[combs[1,]], p2 = pd[combs[2,]], diff = c(diff), p_diff = p_diff)
  }
  lwr <- stats::aggregate(moods$latent1, list(moods$period), 
                   \(x)c(unname(quantile(x, lwr_level))))
  upr <- stats::aggregate(moods$latent1, list(moods$period), 
                   \(x)c(unname(quantile(x, upr_level))))
  names(lwr) <- c("period", "lwr")
  names(upr) <- c("period", "upr")
  cis <- merge(lwr, upr)
  cis <- merge(get_mood(e_orig), cis)
  res <- list(ci = cis)
  if(pw)res[["pw"]] <- res_pw
  return(res)
}

library(testthat)
library(DyadRatios)

# ---- helpers ---------------------------------------------------------------
make_synthetic_data <- function(n_items = 3, n_years = 20,
                                seed = 42, start_year = 1980) {
  set.seed(seed)
  years <- seq(start_year, start_year + n_years - 1)
  true_mood <- 50 + cumsum(rnorm(n_years, 0, 1))

  items <- paste0("item_", letters[seq_len(n_items)])
  rows <- list()
  for (item in items) {
    sign <- sample(c(1, -1), 1)
    noise <- rnorm(n_years, 0, 3)
    index <- 50 + sign * (true_mood - mean(true_mood)) / sd(true_mood) * 8 + noise
    rows[[item]] <- data.frame(
      varname = item,
      date    = as.Date(paste0(years, "-07-01")),
      index   = index,
      n       = sample(800:1200, n_years, replace = TRUE),
      stringsAsFactors = FALSE
    )
  }
  do.call(rbind, rows)
}

# ============================================================================

test_that("extract returns correct structure", {
  dat <- make_synthetic_data()
  res <- extract(dat, n_col = "n", smoothing = FALSE)

  expect_s3_class(res, "extract")
  expect_true(is.numeric(res$mood))
  expect_equal(length(res$mood), res$n_periods)
  expect_true(is.data.frame(res$loadings))
  expect_true(is.data.frame(res$iterations))
  expect_true(is.data.frame(res$periods))
  expect_true("dim1" %in% names(res$loadings))
  expect_equal(nrow(res$loadings), res$n_series)
})

test_that("mood has no NAs", {
  dat <- make_synthetic_data()
  res <- extract(dat, n_col = "n", smoothing = FALSE)
  expect_false(anyNA(res$mood))
})

test_that("smoothing = TRUE runs without error", {
  dat <- make_synthetic_data(n_years = 25)
  expect_no_error(
    extract(dat, n_col = "n", smoothing = TRUE)
  )
})

test_that("n_dim = 2 works and returns two mood vectors", {
  dat <- make_synthetic_data(n_items = 5, n_years = 25)
  res <- extract(dat, n_col = "n", n_dim = 2, smoothing = FALSE)
  expect_equal(length(res$mood),      res$n_periods)
  expect_equal(length(res$mood_dim2), res$n_periods)
  expect_false(anyNA(res$mood))
  expect_false(anyNA(res$mood_dim2))
  expect_true("dim2" %in% names(res$loadings))
})

test_that("variance_explained is in (0, 1]", {
  dat <- make_synthetic_data()
  res <- extract(dat, n_col = "n", smoothing = FALSE)
  expect_true(res$variance_explained > 0 && res$variance_explained <= 1)
})

test_that("loadings are in [-1, 1]", {
  dat <- make_synthetic_data()
  res <- extract(dat, n_col = "n", smoothing = FALSE)
  expect_true(all(abs(res$loadings$dim1) <= 1 + 1e-6))
})

test_that("quarterly aggregation produces correct number of periods", {
  dat <- make_synthetic_data(n_years = 10)
  dat2 <- dat
  dat2$date <- dat$date + 90
  dat2$date[dat2$date > max(dat$date)] <- max(dat$date)
  combined <- rbind(dat, dat2)
  combined$date <- as.Date(paste0(format(combined$date, "%Y-"),
                                  c("01","04","07","10")[ceiling(as.integer(format(combined$date,"%m"))/3)],
                                  "-01"))
  expect_no_error(
    extract(combined, n_col = "n",
            agg_interval = "quarterly", smoothing = FALSE)
  )
})

test_that("start_date / end_date filtering works", {
  dat <- make_synthetic_data(n_years = 30, start_year = 1970)
  res_full <- extract(dat, n_col = "n", smoothing = FALSE)
  res_trim <- extract(dat, n_col = "n", smoothing = FALSE,
                      start_date = "1980-01-01",
                      end_date   = "1989-12-31")
  expect_true(res_trim$n_periods < res_full$n_periods)
  expect_true(res_trim$settings$start_date >= as.Date("1980-01-01"))
})

test_that("print and summary do not error", {
  dat <- make_synthetic_data()
  res <- extract(dat, n_col = "n", smoothing = FALSE)
  expect_no_error(print(res))
  expect_no_error(summary(res))
})

test_that("missing n_col uses default weight of 1000", {
  dat <- make_synthetic_data()
  dat$n <- NULL
  res <- expect_no_error(extract(dat, smoothing = FALSE))
  expect_s3_class(res, "extract")
})

# ============================================================================
# boot_dr tests
# ============================================================================

test_that("boot_dr returns a list with correct structure", {
  dat  <- make_synthetic_data()
  res  <- extract(dat, n_col = "n", smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 1)
  expect_s3_class(boot, "boot_dr")
  expect_true(all(c("estimates", "samples") %in% names(boot)))
  expect_s3_class(boot$estimates, "data.frame")
  expect_true(is.matrix(boot$samples))
  expect_true(all(c("period", "year", "mood", "lower", "upper") %in% names(boot$estimates)))
})

test_that("boot_dr estimates has one row per period", {
  dat  <- make_synthetic_data(n_years = 15)
  res  <- extract(dat, n_col = "n", smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 2)
  expect_equal(nrow(boot$estimates), res$n_periods)
})

test_that("boot_dr samples matrix has correct dimensions", {
  dat  <- make_synthetic_data()
  res  <- extract(dat, n_col = "n", smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 2)
  expect_equal(nrow(boot$samples), res$n_periods)
  expect_equal(ncol(boot$samples), attr(boot, "R"))
})

test_that("boot_dr mood column matches original extract", {
  dat  <- make_synthetic_data()
  res  <- extract(dat, n_col = "n", smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 3)
  expect_equal(boot$estimates$mood, res$mood)
})

test_that("boot_dr confidence bounds bracket the point estimate", {
  dat  <- make_synthetic_data()
  res  <- extract(dat, n_col = "n", smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 50, seed = 4)
  expect_true(all(boot$estimates$lower <= boot$estimates$mood + 1e-9))
  expect_true(all(boot$estimates$upper >= boot$estimates$mood - 1e-9))
})

test_that("boot_dr R attribute records successful replications", {
  dat  <- make_synthetic_data()
  res  <- extract(dat, n_col = "n", smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 30, seed = 5)
  expect_true(attr(boot, "R") <= 30)
  expect_true(attr(boot, "R") > 0)
})

test_that("plot.boot_dr runs without error", {
  dat  <- make_synthetic_data()
  res  <- extract(dat, n_col = "n", smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 6)
  expect_no_error(plot(boot))
})

# ============================================================================
# boot_dr two-dimension tests
# ============================================================================

test_that("boot_dr n_dim=2 returns samples_dim2 matrix", {
  dat  <- make_synthetic_data(n_items = 5, n_years = 25)
  res  <- extract(dat, n_col = "n", n_dim = 2, smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 7)
  expect_true("samples_dim2" %in% names(boot))
  expect_true(is.matrix(boot$samples_dim2))
  expect_equal(nrow(boot$samples_dim2), res$n_periods)
  expect_equal(ncol(boot$samples_dim2), attr(boot, "R"))
})

test_that("boot_dr n_dim=2 estimates has dim2 CI columns", {
  dat  <- make_synthetic_data(n_items = 5, n_years = 25)
  res  <- extract(dat, n_col = "n", n_dim = 2, smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 8)
  expect_true(all(c("mood_dim2", "lower_dim2", "upper_dim2") %in% names(boot$estimates)))
  expect_false(anyNA(boot$estimates$mood_dim2))
})

test_that("boot_dr n_dim=2 mood_dim2 matches original extract", {
  dat  <- make_synthetic_data(n_items = 5, n_years = 25)
  res  <- extract(dat, n_col = "n", n_dim = 2, smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 9)
  expect_equal(boot$estimates$mood_dim2, res$mood_dim2)
})

test_that("boot_dr n_dim=2 dim2 CI bounds bracket point estimate", {
  dat  <- make_synthetic_data(n_items = 5, n_years = 25)
  res  <- extract(dat, n_col = "n", n_dim = 2, smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 50, seed = 10)
  expect_true(all(boot$estimates$lower_dim2 <= boot$estimates$mood_dim2 + 1e-9))
  expect_true(all(boot$estimates$upper_dim2 >= boot$estimates$mood_dim2 - 1e-9))
})

test_that("boot_dr n_dim=2 n_dim attribute is 2", {
  dat  <- make_synthetic_data(n_items = 5, n_years = 25)
  res  <- extract(dat, n_col = "n", n_dim = 2, smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 11)
  expect_equal(attr(boot, "n_dim"), 2L)
})

test_that("plot.boot_dr dim=2 runs without error", {
  dat  <- make_synthetic_data(n_items = 5, n_years = 25)
  res  <- extract(dat, n_col = "n", n_dim = 2, smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 12)
  expect_no_error(plot(boot, dim = 2))
})

test_that("plot.boot_dr dim=2 errors on 1-dim object", {
  dat  <- make_synthetic_data()
  res  <- extract(dat, n_col = "n", smoothing = FALSE)
  boot <- boot_dr(res, dat, R = 20, seed = 13)
  expect_error(plot(boot, dim = 2))
})

#' Summary Method for extract Objects
#' 
#' Prints a summary from objects estimated with the the extract function. 
#' 
#' @param object An object of class "extract", typically a result of the extract function.
#' @param ... Other arguments to be passed down (currently unused).
#' @return A data frame with variables for the question name, number of cases, loading as well as mean and standard deviation of the series. 
#' @usage NULL
#' @method summary extract
#' @export
#' @examples
#' data(jennings)
#' dr_out <- extract(varname = jennings$variable, 
#'                   date = jennings$date, 
#'                   index = jennings$value, 
#'                   ncases = jennings$n, 
#'                   begindt = min(jennings$date), 
#'                   enddt = max(jennings$date), 
#'                   npass=1)
#' summary(dr_out)
summary.extract <- function(object, ...) {
  T=object$T
  nvar=object$nvar
  dim<- object$dimensions
  vn<- c(object$varname,"Variable Name")
  vn<- format(vn,justify="right")
  nc<- format(object$N,justify="right")
  ld<- format(object$loadings1,digits=3,justify="right")
  mean<- format(object$means,digits=6,justify="right")
  sd<- format(object$std.deviations,digits=6,justify="right")
  cat("Variable Loadings and Descriptive Information: Dimension 1\n")
  cat(paste(vn[nvar+1],"Cases","Loading","   Mean ","Std Dev","\n"))
  for (v in 1:nvar) {
    cat(paste(vn[v],"  ",nc[v]," ",ld[v],mean[v],sd[v],"\n"))
    }
  if (dim == 2) {
    ld<- format(object$loadings2,digits=3,justify="right")
    cat("\nVariable Loadings and Descriptive Information: Dimension 2\n")
    cat(paste(vn[nvar+1],"Cases","Loading","   Mean ","Std Dev","\n"))
    for (v in 1:nvar) {
      cat(paste(vn[v],"  ",nc[v]," ",ld[v],mean[v],sd[v],"\n"))
    }
  }
}

#' dyadratios: Dyad Ratios Algorithm for Latent Public Opinion Estimation
#'
#' @description
#' Implements the Dyad Ratios algorithm (Stimson 1991) for estimating latent
#' public mood from a collection of time-series survey marginals.  The
#' computationally intensive estimation loop is written in C++ (via Rcpp) and
#' is a faithful translation of James Stimson's original 'MCalc' program.  The R
#' layer handles data ingestion, temporal aggregation, result formatting, and
#' visualisation.
#'
#' @section Main functions:
#' \describe{
#'   \item{\code{\link{extract}}}{Run the algorithm on a data frame.}
#' }
#'
#' @section Input data format:
#' The primary input is a data frame where each row is one survey marginal:
#' \itemize{
#'   \item A column identifying the opinion \strong{variable} (issue series).
#'   \item A \strong{date} column (any format coercible by \code{as.Date}).
#'   \item An \strong{index} column: the survey proportion or percentage.
#'   \item An optional \strong{n} column: number of respondents (used as
#'         weight; defaults to 1000 if omitted).
#' }
#'
#' @references
#' Stimson, J. A. (1991). \emph{Public Opinion in America: Moods, Cycles, and
#' Swings}. Boulder, CO: Westview Press.
#'
#' Stimson, J. A. (1999). \emph{Public Opinion in America}, 2nd ed. Westview.
#'
#' @docType package
#' @name dyadratios-package
#' @useDynLib DyadRatios, .registration = TRUE
#' @importFrom Rcpp evalCpp
#' @importFrom stats rbinom quantile
#' @importFrom graphics polygon
"_PACKAGE"

#' Dyad Ratios Algorithm
#'
#' Estimates the Dyad Ratios Algorithm for constructing latent time series from
#' survey-research marginals.
#'
#' @name extract
#' @aliases extract
#' @param varname String giving the name of the input series to be smoothed.
#'   This should identify similar or comparable values in the series. Values in
#'   the series that have the same \code{varname} will be assumed to come from
#'   the same source.
#' @param date ISO numeric representation of the date the survey was in the
#'   field (usually the start, end, or median date).
#' @param index Numeric value of the series. It might be a percent or proportion
#'   responding in a single category (e.g., the approve response in presidential
#'   approval) or some multi-response summary. For ease of interpretation,
#'   polarity should be the same for all items.
#' @param ncases Number of cases (e.g., sample size) of the survey. This provides
#'   differential weighting for the values. Setting this to \code{NULL} or
#'   leaving it blank will weight each value equally.
#' @param unit Aggregation period—one of \sQuote{D} (daily), \sQuote{M}
#'   (monthly), \sQuote{Q} (quarterly), \sQuote{A} (annual), or \sQuote{O}
#'   (multi-year aggregation).
#' @param mult Number of years, only used if \code{unit} is \sQuote{O}.
#' @param begindt Beginning date of the analysis. Defaults to earliest date in
#'   the dataset. Should be specified with \code{lubridate::ymd()}.
#' @param enddt Ending date for the analysis. Defaults to the latest date in the
#'   data.
#' @param npass Not yet implemented.
#' @param smoothing Logical. Specifies whether exponential smoothing is applied
#'   to the intermediate estimates during the iterative solution process.
#'   Defaults to \code{TRUE}.
#' @param endmonth Ending month of the analysis.
#'
#' @details In previous versions of the algorithm, especially in its original incarnation as `Wcalc.exe`, getting the dates right was really important.  They needed to be in precisely the right format and the
#' data needed to be ordered by date.  In this version of the `extract()` function, any R date class is fine whether it is a `Date` class or `POSIX*` class.  The function uses `is.Date` from the 
#' `lubridate` package to check if it is a `Date` class and `as.Date()` from R's `base` package to coerce it to a `Date` class if it is not.  Further, the series are ordered by date in the function 
#' as a matter of course, so it is not necessary that they be in ascending ordered of date in the input data. 
#'
#' @return A list with components:
#' \itemize{
#'   \item \code{call}: The initial call to `extract()`. 
#'   \item \code{T}: Number of aggregation periods.
#'   \item \code{nvar}: Number of series used in the analysis.
#'   \item \code{unit}: The aggregation period.
#'   \item \code{dimensions}: Number of dimensions estimated (1 or 2).
#'   \item \code{period}: List of aggregation periods.
#'   \item \code{varname}: List in order of the variables used in the analysis.
#'   \item \code{N}: Number of non-missing observations for each series.
#'   \item \code{wtdmean}: The weighted mean of the input variables. 
#'   \item \code{wtstd}: The weighted standard deviation of the input series. 
#'   \item \code{means}: Item descriptive information.
#'   \item \code{std.deviations}: Item descriptive information.
#'   \item \code{setup1}: Basic information about the options and iterative solution for the first dimension.
#'   \item \code{setup2}: Basic information about the options and iterative solution for the second dimension, if applicable.
#'   \item \code{loadings1}: Item–scale correlations from the first dimension of the final solution.
#'     Their square is the validity estimate used in weighting.
#'   \item \code{loadings2}: Item–scale correlations from the second dimension of the final solution, if applicable.
#'   \item \code{latent1}: Estimated time series for first dimension.
#'   \item \code{latent2}: Estimated time series for second dimension, if applicable.
#'   \item \code{hist}: Data frame with iteration history, including convergence statistics.
#'   \item \code{totalvar}: Total variance in the data that could be explained by the estimated dimensions.
#'   \item \code{var_exp}: Data frame with variance explained and proportion of variance explained for each dimension.
#'   \item \code{dropped}: Data frame listing any series that were dropped from the analysis due to insufficient data.
#'   \item \code{smoothing}: Logical indicating whether smoothing was applied during the iterative solution process.
#'   \item \code{ms_list}: List of means and variances for standardizing the input series for the bootstrap. 
#' }
#'
#' @references
#' Stimson, J. A. (2018).
#' \sQuote{The Dyad Ratios Algorithm for Estimating Latent Public Opinion:
#' Estimation, Testing, and Comparison to Other Approaches},
#' \emph{Bulletin of Sociological Methodology/Bulletin de Méthodologie
#' Sociologique}, 137–138(1), 201–218. \doi{10.1177/0759106318761614}
#'
#' @importFrom stats cor lm optim sd var
#' @importFrom graphics legend lines par plot 
#' @importFrom lubridate ymd year month day quarter is.Date
#' 
#' @usage NULL
#' @export
#'
#' @examples
#' data(jennings)
#' dr_out <- extract(varname = jennings$variable, 
#'                   date = jennings$date, 
#'                   index = jennings$value, 
#'                   ncases = jennings$n, 
#'                   begindt = min(jennings$date), 
#'                   enddt = max(jennings$date), 
#'                   npass=1)
#' summary(dr_out)
#' 
extract <- function(varname,date,index,ncases=NULL,unit="A",mult=1,begindt=NA,enddt=NA,npass=1,smoothing=TRUE,endmonth=12) {
  if(!is.Date(date))date <- as.Date(date)
  tmp <- data.frame(varname = varname, date = date, index = index, ncases = ncases)
  sp_tmp <- split(tmp, tmp$varname)
  for(i in seq_along(sp_tmp)) {
    sp_tmp[[i]] <- sp_tmp[[i]][order(sp_tmp[[i]]$date), ]
  }
  tmp <- do.call(rbind, sp_tmp)
  varname <- tmp$varname
  index <- tmp$index
  date <- tmp$date
  ncases <- tmp$ncases
  formula<-match.call(extract)
  nrecords<- length(varname)
  if (is.null(ncases)) ncases<- rep(0,nrecords)
  moddate <- date #create temporary date vector, leaving original unmodified
  if ((unit=="A" || unit=="O") && endmonth<12) {
    for (i in 1:nrecords) { #first loop through raw data file
      month<- month(moddate[i])
      year<- year(moddate[i])
      if (month>endmonth) moddate[i]<- ymd(paste(year, "01", "01", sep="-")) #modified date become 1/1 of next year
    } #end loop through data
  } # end if

  if (is.na(begindt)) minper<-month(min(moddate)) else minper<-month(begindt)
  if (is.na(begindt)) miny<-year(min(moddate)) else miny<-year(begindt)
  if (is.na(begindt)) minday<-day(min(moddate)) else minday<-day(begindt)
  if (is.na(enddt)) maxper<-month(max(moddate)) else maxper<-month(enddt)
  if (is.na(enddt)) maxy<-year(max(moddate)) else maxy<-year(enddt)
  if (is.na(enddt)) maxday<-day(max(moddate)) else maxday<-day(enddt)
  mindate <- lubridate::ymd(paste(miny, minper, minday, sep="-"))
  maxdate <- lubridate::ymd(paste(maxy, maxper, maxday, sep="-"))

  w <- which(moddate >= mindate & moddate <=maxdate)
  if(length(w) == 0){
    stop("No surveys within date range. \n")
  }
  varname <- varname[w]
  index <- index[w]
  if(!is.null(ncases)){
    ncases <- ncases[w]
  }
  moddate <- moddate[w]
  date <- date[w]
  
  if (unit=="Q") {
    minper<- as.integer((minper-1)/3)+1
    maxper<- as.integer((maxper-1)/3)+1
  }
  # mindate<- ISOdate(miny,minper,minday,0,0,0,tz="GMT")
  # maxdate<- ISOdate(maxy, maxper, maxday,0,0,0,tz="GMT") #86400=24*60*60

  
  
  #SETCONS:
  latent<- numeric(1)
  aggratio<- 0
  fb<- 1 #initialize
  auto<- "start"  #meaningless value
  alpha<- 1
  alpha1<- 1
  pass<- 1
  holdtola<- 0.001
  tola<- holdtola
  iter<- 0
  lastconv<- 99999
  wtmean<- 0 #for it=1
  wtstd<- 1
  fract<- 1
  iter_hist <- NULL
  dropped <- NULL
  if (unit=="A") {
    nperiods<- maxy-miny+1
    aggratio<- 1
    months<- 12
  }
  if (unit=="O") {
    years<- mult
    months<- years*12
    aggratio<- 2
    odd<- (maxy-miny+1) %% mult  #mod
    nperiods=as.integer((maxy-miny)/mult) + odd
  }
  if (unit=="M") {
    fract<- 100
    nperiods<- (maxy-miny)*12
    nperiods<- nperiods-12 + (12-minper+1) + maxper
    aggratio<- 1/12
    months<- 1
  }
  if (unit=="Q") {
    aggratio<- 1/4
    months<- 3
    nperiods<- as.integer((maxy-miny)/aggratio)
    nperiods<- nperiods-4 + (4-minper+1) + maxper
    fract<- 10
  }
  if (unit=="D") {
    months=1
    nperiods<- (as.integer(maxdate)-as.integer(mindate)) + 1 
  }

  arinv<- 1/aggratio
  aggratio<- months/12
  nrecords<- length(index)

#HERE WE SET UP FUNDAMENTAL DIMENSIONS AND DECLARE VECTORS
  if (fb != 2)  mood<- array(dim=c(3,nperiods))
  vfac<- factor(varname) #make a factor vector
  vlev<- levels(vfac)    #find unique categories
  nvar<- length(vlev)    #how many are there?, includes unusable series
  valid <- numeric(nvar)
  csign <- numeric(nvar)
  vl <- character(nvar)
  r<- numeric(nvar)
  oldr<- rep(1,nvar) # r=1 for all v initially

  issue<- array(dim=c(nperiods,nvar))
  count<- numeric(nperiods)
  vl<- numeric(nvar)
  period<- numeric(nperiods)
  converge<- 0
  evalue<- 0
  evalue1 <- 0
  evalue2 <- NULL
  # create numeric variable period, eg, yyyy.0m 
  if (unit=="D") {
    period<-seq(1:nperiods) 
    } else {
  if (months >= 12) {
    for (l in 1:nperiods) { 
      p <- (l - 1) * aggratio
      period[l] <- miny + p
      } #next l
   } else {
     y <- 0
     i <- 0
     my <- miny
     if (minper == 1)  my <- my - 1
     for (l in 1:nperiods) { 
       i<- 1 + ((l-1) %% arinv)
       mq <- minper + i - 1
       mq<- 1 + ((mq-1) %% arinv) 
       if (mq == 1)  y <- y + 1 #first month or quarter, increment year
       period[l] <- my + y + mq / fract
     } # end for
   } #end else
  } # end if


  agglist<- aggregate(varname,moddate,index,ncases,mindate,maxdate,nperiods,nvar,aggratio,unit,miny,minper) # call aggregate to produce issue matrix
  vl<- agglist$lab #extract two elements of the list from aggregate call
  issue<- agglist$iss
  rm(agglist) #don't need this anymore

  #NOW REDUCE ISSUE MATRIX TO ELIMINATE UNUSABLE SERIES (WN<2)
  ndrop<- 0
  nissue<- numeric(nperiods)
  std<- numeric(nperiods)
  for (v in 1:nvar) {
    std[v]<- 0 #default
    nissue[v]<- sum(!is.na(issue[,v])) #criterion is 2 cases for npass=1 or 3 for npass=2
    if (nissue[v]>npass) std[v]<- sqrt(var(issue[,v],na.rm=TRUE)) #this is just a test for variance >0
    if (std[v]<.001) {  #case dropped if std uncomputable (NA) or actually zero (constant)
      ndrop<- ndrop+1
      dropped <- rbind(dropped, data.frame(Dropped = vl[v], N_remain = nissue[v]))
    }
    }
  nvarold<- nvar
  nvar<- nvar-ndrop
  pointer<- 1
  found<- FALSE
  
  for (v in 1:nvar) { #now reduced nvar
    while (found==FALSE && pointer<=nvarold) { #find first valid column and push down
      if (std[pointer]>.001) { #good case, transfer
         issue[,v]<- issue[,pointer]
         vl[v]<- vl[pointer]
         pointer<- pointer+1
         found<- TRUE
      } else {
        pointer<- pointer+1 #bad case, increment pointer
      } #end if
    } #end while
    found<- FALSE
  } #for
  length(vl)<- nvar #reduce  
  length(issue)<- nperiods*nvar  #chop off unused columns
  attr(issue,"dim")<- c(nperiods,nvar)
  N<- numeric(nvar)

#export<<-list(nperiods,nvar,issue)

  ms_list <- vector(mode = "list", length=npass)
  for (pass in 1:npass) { #newpass: RESTART FOR SECOND DIMENSION CASE
    if (pass == 2) { #reset iteration control parameters
      iter <- 0
      tola = holdtola
      lastconv <- 99999
      converge<- lastconv
      conv<- converge
    } else {
      av<- numeric(nvar)
      std<- numeric(nvar)
#      ngood<- 0
      for (v in 1:nvar) { #compute av and std by issue nvar now reduced to good cases
        wn<- as.integer(nperiods-sum(is.na(issue[,v])))
        av[v] <- mean(issue[,v],na.rm=TRUE)
        std[v]<- sqrt(var(issue[,v],na.rm=TRUE) * ((wn - 1)/wn)) #convert to population standard deviation
        issue[,v]<- 100 + 10 * (issue[,v] - av[v])/std[v]  #standardize
#        ngood<- ngood+1
      }#end for
    ms_list[[pass]] <- list(av=av, std=std)
    }
    #READY FOR ESTIMATION, SET UP AND PRINT OPTIONS INFO     
    out<- as.character(10) #initial length only
    setup <- list(start = miny, end = maxy, nperiods = nperiods, used_series = nvar, unused_series = ndrop, smoothiing=smoothing)
    if(!is.null(minper)){
      setup$minper <- minper
      setup$maxper <- maxper
    }
    for (p in 1:nperiods) {
      count[p]<- sum(!is.na(issue[p,]))
    }
    valid <- rep(1, times = nvar)
    csign <- rep(1, times = nvar)
    auto <- "y"              #iterative estimation on by default
    quit <- 0                #false implies go ahead and estimate

    while (iter == 0 || converge > tola) {   #MASTER CONTROL LOOP WHICH ITERATES UNTIL SOLUTION REACHED

      for (fb in 1:2) { #    MASTER fb LOOP       fb=1 is forward, 2 backward 
        dominate.out<- dominate(fb,issue,nperiods,nvar,mood,valid,smoothing,alpha, csign)  #master estimation routine
        alpha1<- dominate.out$alpha1
        alpha<- dominate.out$alpha
        mood[fb,]<- dominate.out$latent
        } #next fb

    fb <- 3 #average mood from here on
    for (p in 1:nperiods) { #    AVERAGE
      mood[fb, p] <- (mood[1, p] + mood[2, p]) / 2
    } #next p
    moodmean<-mean(mood[3,])
    sdmood<-sd(mood[3,])
    for (p in 1:nperiods) {  #PLACEMENT OF THIS LOOP MAY NOT BE RIGHT
      mood[fb,p] <- ((mood[fb,p] - moodmean) * wtstd / sdmood) + wtmean
    } #end for

    iter <- iter + 1 
    if (auto == "y") r<- iscorr(issue,mood) else auto <- "y"   #recompute correlations

    wtmean<- 0
    wtstd<- 0
    vsum<- 0
    goodvar<- 0
    converge<- 0 #start off default
    evalue<- 0
    totalvar<- 0

    for (v in 1:nvar) {
      wn<- nperiods-sum(is.na(issue[,v]))
      if (!is.na(sign(r[v]))) csign[v]<- sign(r[v])
      #if (!is.na(sign(r[v]))) csign[v]<<- sign(r[v])
      wn<- nperiods-sum(is.na(issue[,v]))
      if (wn>1) { #sum over variables actually used
        vratio <- wn / nperiods
        evalue <- evalue + vratio * r[v]^2
        totalvar <- totalvar + vratio
      } #end if
      evalue1 <- evalue
      totalvar1 <- totalvar
      #convergence tests
      if (wn > 3) {
       conv <- abs(r[v] - oldr[v])      #conv is convergence test for item=v
       conv <- conv * (wn / nperiods)                #weight criterion by number of available periods
       if (conv > converge)  converge <- conv        #converge is the global max of conv
      } #end if
      if (!is.na(r[v])) oldr[v] <- r[v]
      if (!is.na(r[v])) valid[v] <- r[v]^2
      if (!is.na(av[v])) wtmean <- wtmean + av[v] * valid[v]
      if (!is.na(std[v])) wtstd <- wtstd + std[v] * valid[v]
      if (!is.na(r[v])) vsum <- vsum + valid[v]
    } #end v loop

    if (vsum > 0)  wtmean <- wtmean / vsum
    if (vsum > 0)  wtstd <- wtstd / vsum
    if (pass == 1) {
      mean1 <- wtmean
      std1 <- wtstd
      e1=evalue
    } else {
      wtmean <- mean1
      wtstd <- std1 #*unexp
    } #end if
    fbcorr <- cor(mood[1,],mood[2,]) #fnfrontback 
    if (quit != 1) {
      cv<- format(round(converge,4),nsmall=4) 
      itfmt<-format(round(iter),justify="right",length=4)
      iter_hist <- rbind(iter_hist, data.frame(Dimension = paste0("Dimension ", pass), Iter = itfmt, Convergence = cv, Criterion = tola, 
                                                Reliability = fbcorr, Alphaf = alpha1, Alphab = alpha))
      }
    if (converge > lastconv)  tola <- tola * 2
    lastconv <- converge

    auto = "y"  #skip corr on iter=1, set auto on

    if (iter >= 50) break #get out of while loop
    } #END MASTER WHILE ITERATION CONTROL LOOP

    if (auto == "y" && converge<tola) { #IF WE REACH THIS CODE WE HAVE A FINAL SOLUTION TO BE REPORTED
    if (pass == 1) out1<- out #hold output for 2 dimensional solution
    auto <- "Q"
    quit <- 1                #flag solution reached, last time through
    r<- iscorr(issue,mood)   #final iteration correlations
    if (pass == 1) r1<- r #hold correlations for 2 dimensional solution

    if (pass > 1) {
      unexp <- totalvar 
      totalvar <- unexp * totalvar
      evalue <- evalue * unexp
    } #    end if

    if (pass == 1) {
      expprop <- evalue / totalvar
      tot1 <- totalvar
      tot_var <- tot1
      expprop1 <- expprop
    } else {
      erel <- evalue / totalvar          #% exp relative
      totalvar <- (1 - expprop) * tot1   #true var=original var discounted by %exp
      evalue <- erel * totalvar          #rescale to retain %exp relationship
      expprop <- evalue / tot1           #now reduce eral to expprop
      evalue2 <- evalue
      expprop2 <- expprop
    } #    end if

    for (v in 1:nvar) {
      N[v]<- sum(!is.na(issue[,v]))
      }
    var.out<- list(varname=vl,loadings=r,means=av,std.deviations=std)
    if (pass !=  2 && npass>1) {
      for (v in 1:nvar) { 
        valid[v] <- 0               #reset all, regmoodissue will set good=1
        if (csign[v] != 0)  issue[,v]<- residmi(issue,v,mood)   #regmoodissue()
       } #v loop
    }  # if
    #begin prn output routine # mood[fb,] is now our estimate,    WHAT ABOUT A SECOND DIMENSION
    
    var_exp <- data.frame(Dimension =  "Dimension 1", 
               Variance = evalue1, 
               Proportion = expprop1)
    if(!is.null(evalue2)){
      var_exp <- rbind(var_exp, 
                       data.frame(Dimension = "Dimension 2", 
                                  Variance = evalue2, 
                                  Proportion = expprop2))
    }
    
    latent<- mood[fb,] #vector holds values for output
    if (pass == 1) latent1<- latent #hold first dimension
    #for Zelig output
    if (npass==1) {
      extract.out<- list(call=formula,
                         T=nperiods,
                         nvar=nvar,
                         unit=unit,
                         dimensions=npass,
                         period=period,
                         varname=vl,
                         N=N,
                         wtdmean=wtmean,
                         wtdstd=wtstd,
                         means=av,
                         std.deviations=std,
                         setup1=out1,
                         loadings1=r1,
                         latent1=latent1, 
                         hist = iter_hist, 
                         totalvar = totalvar1, 
                         var_exp = var_exp, 
                         dropped=dropped, 
                         smoothing = smoothing, 
                         ms_list = ms_list)
    } else {
    extract.out<- list(call=formula,
                       T=nperiods,
                       nvar=nvar,
                       unit=unit,
                       dimensions=npass,
                       period=period,
                       varname=vl,
                       N=N,
                       wtdmean=wtmean,
                       wtdstd=wtstd,
                       means=av,
                       std.deviations=std,
                       setup1=out1,
                       loadings1=r1,
                       latent1=latent1,
                       setup2=out,
                       loadings2=r,
                       latent2=latent, 
                       hist = iter_hist,
                       totalvar = totalvar1, 
                       var_exp = var_exp, 
                       dropped=dropped, 
                       smoothing = smoothing, 
                       ms_list = ms_list)
    }
    } #end if auto="y" 
  } #end of for pass=1,2 loop 

  # par(col=1) #reset on termination
  class(extract.out)<- "extract"
  return(extract.out)
  }

#' Extract for bootstrap includes arguments for means and standard deviations for standardizing
#' @noRd
extract_bs <- function(varname,date,index,ncases=NULL,unit="A",mult=1,begindt=NA,enddt=NA,npass=1,smoothing=TRUE,endmonth=12, wm, ws) {
  if(!is.Date(date))date <- as.Date(date)
  tmp <- data.frame(varname = varname, date = date, index = index, ncases = ncases)
  sp_tmp <- split(tmp, tmp$varname)
  for(i in seq_along(sp_tmp)) {
    sp_tmp[[i]] <- sp_tmp[[i]][order(sp_tmp[[i]]$date), ]
  }
  tmp <- do.call(rbind, sp_tmp)
  varname <- tmp$varname
  index <- tmp$index
  date <- tmp$date
  ncases <- tmp$ncases
  formula<-match.call(extract_bs)
  csign <- NULL
  nrecords<- length(varname)
  if (is.null(ncases)) ncases<- rep(0,nrecords)
  moddate <- date #create temporary date vector, leaving original unmodified
  if ((unit=="A" || unit=="O") && endmonth<12) {
    for (i in 1:nrecords) { #first loop through raw data file
      month<- month(moddate[i])
      year<- year(moddate[i])
      if (month>endmonth) moddate[i]<- ymd(paste(year, "01", "01", sep="-")) #modified date become 1/1 of next year
    } #end loop through data
  } # end if
  
  if (is.na(begindt)) minper<-month(min(moddate)) else minper<-month(begindt)
  if (is.na(begindt)) miny<-year(min(moddate)) else miny<-year(begindt)
  if (is.na(begindt)) minday<-day(min(moddate)) else minday<-day(begindt)
  if (is.na(enddt)) maxper<-month(max(moddate)) else maxper<-month(enddt)
  if (is.na(enddt)) maxy<-year(max(moddate)) else maxy<-year(enddt)
  if (is.na(enddt)) maxday<-day(max(moddate)) else maxday<-day(enddt)
  if (unit=="Q") {
    minper<- as.integer((minper-1)/3)+1
    maxper<- as.integer((maxper-1)/3)+1
  }
  # mindate<- ISOdate(miny,minper,minday,0,0,0,tz="GMT")
  # maxdate<- ISOdate(maxy, maxper, maxday,0,0,0,tz="GMT") #86400=24*60*60
  
  mindate <- lubridate::ymd(paste(miny, minper, minday, sep="-"))
  maxdate <- lubridate::ymd(paste(maxy, maxper, maxday, sep="-"))
  
  
  #SETCONS:
  latent<- numeric(1)
  aggratio<- 0
  fb<- 1 #initialize
  auto<- "start"  #meaningless value
  alpha<- 1
  alpha1<- 1
  pass<- 1
  holdtola<- 0.001
  tola<- holdtola
  iter<- 0
  lastconv<- 99999
  wtmean<- 0 #for it=1
  wtstd<- 1
  fract<- 1
  iter_hist <- NULL
  dropped <- NULL
  if (unit=="A") {
    nperiods<- maxy-miny+1
    aggratio<- 1
    months<- 12
  }
  if (unit=="O") {
    years<- mult
    months<- years*12
    aggratio<- 2
    odd<- (maxy-miny+1) %% mult  #mod
    nperiods=as.integer((maxy-miny)/mult) + odd
  }
  if (unit=="M") {
    fract<- 100
    nperiods<- (maxy-miny)*12
    nperiods<- nperiods-12 + (12-minper+1) + maxper
    aggratio<- 1/12
    months<- 1
  }
  if (unit=="Q") {
    aggratio<- 1/4
    months<- 3
    nperiods<- as.integer((maxy-miny)/aggratio)
    nperiods<- nperiods-4 + (4-minper+1) + maxper
    fract<- 10
  }
  if (unit=="D") {
    months=1
    nperiods<- (as.integer(maxdate)-as.integer(mindate)) + 1 
  }
  
  arinv<- 1/aggratio
  aggratio<- months/12
  nrecords<- length(index)
  
  #HERE WE SET UP FUNDAMENTAL DIMENSIONS AND DECLARE VECTORS
  if (fb != 2)  mood<- array(dim=c(3,nperiods))
  vfac<- factor(varname) #make a factor vector
  vlev<- levels(vfac)    #find unique categories
  nvar<- length(vlev)    #how many are there?, includes unusable series
  valid<- numeric(nvar)
  csign<- numeric(nvar)
  vl<- character(nvar)
  r<- numeric(nvar)
  oldr<- rep(1,nvar) # r=1 for all v initially
  
  issue<- array(dim=c(nperiods,nvar))
  count<- numeric(nperiods)
  vl<- numeric(nvar)
  period<- numeric(nperiods)
  converge<- 0
  evalue<- 0
  evalue1 <- 0
  evalue2 <- NULL
  # create numeric variable period, eg, yyyy.0m 
  if (unit=="D") {
    period<-seq(1:nperiods) 
  } else {
    if (months >= 12) {
      for (l in 1:nperiods) { 
        p <- (l - 1) * aggratio
        period[l] <- miny + p
      } #next l
    } else {
      y <- 0
      i <- 0
      my <- miny
      if (minper == 1)  my <- my - 1
      for (l in 1:nperiods) { 
        i<- 1 + ((l-1) %% arinv)
        mq <- minper + i - 1
        mq<- 1 + ((mq-1) %% arinv) 
        if (mq == 1)  y <- y + 1 #first month or quarter, increment year
        period[l] <- my + y + mq / fract
      } # end for
    } #end else
  } # end if
  
  
  agglist<- aggregate(varname,moddate,index,ncases,mindate,maxdate,nperiods,nvar,aggratio,unit,miny,minper) # call aggregate to produce issue matrix
  vl<- agglist$lab #extract two elements of the list from aggregate call
  issue<- agglist$iss
  rm(agglist) #don't need this anymore
  
  #NOW REDUCE ISSUE MATRIX TO ELIMINATE UNUSABLE SERIES (WN<2)
  ndrop<- 0
  nissue<- numeric(nperiods)
  std<- numeric(nperiods)
  for (v in 1:nvar) {
    std[v]<- 0 #default
    nissue[v]<- sum(!is.na(issue[,v])) #criterion is 2 cases for npass=1 or 3 for npass=2
    if (nissue[v]>npass) std[v]<- sqrt(var(issue[,v],na.rm=TRUE)) #this is just a test for variance >0
    if (std[v]<.001) {  #case dropped if std uncomputable (NA) or actually zero (constant)
      ndrop<- ndrop+1
      dropped <- rbind(dropped, data.frame(Dropped = vl[v], N_remain = nissue[v]))
    }
  }
  nvarold<- nvar
  nvar<- nvar-ndrop
  pointer<- 1
  found<- FALSE
  
  for (v in 1:nvar) { #now reduced nvar
    while (found==FALSE && pointer<=nvarold) { #find first valid column and push down
      if (std[pointer]>.001) { #good case, transfer
        issue[,v]<- issue[,pointer]
        vl[v]<- vl[pointer]
        pointer<- pointer+1
        found<- TRUE
      } else {
        pointer<- pointer+1 #bad case, increment pointer
      } #end if
    } #end while
    found<- FALSE
  } #for
  length(vl)<- nvar #reduce  
  length(issue)<- nperiods*nvar  #chop off unused columns
  attr(issue,"dim")<- c(nperiods,nvar)
  N<- numeric(nvar)
  
  #export<<-list(nperiods,nvar,issue)
  
  ms_list <- vector(mode = "list", length=npass)
  for (pass in 1:npass) { #newpass: RESTART FOR SECOND DIMENSION CASE
    if (pass == 2) { #reset iteration control parameters
      iter <- 0
      tola = holdtola
      lastconv <- 99999
      converge<- lastconv
      conv<- converge
    } else {
      av<- numeric(nvar)
      std<- numeric(nvar)
      #      ngood<- 0
      for (v in 1:nvar) { #compute av and std by issue nvar now reduced to good cases
        wn<- as.integer(nperiods-sum(is.na(issue[,v])))
        av[v] <- mean(issue[,v],na.rm=TRUE)
        std[v]<- sqrt(var(issue[,v],na.rm=TRUE) * ((wn - 1)/wn)) #convert to population standard deviation
        issue[,v]<- 100 + 10 * (issue[,v] - av[v])/std[v]  #standardize
        #        ngood<- ngood+1
      }#end for
      ms_list[[pass]] <- list(av=av, std=std)
    }
    #READY FOR ESTIMATION, SET UP AND PRINT OPTIONS INFO     
    out<- as.character(10) #initial length only
    setup <- list(start = miny, end = maxy, nperiods = nperiods, used_series = nvar, unused_series = ndrop, smoothiing=smoothing)
    if(!is.null(minper)){
      setup$minper <- minper
      setup$maxper <- maxper
    }
    for (p in 1:nperiods) {
      count[p]<- sum(!is.na(issue[p,]))
    }
    valid<- rep(1,times=nvar)
    csign<- rep(1,times=nvar)
    auto <- "y"              #iterative estimation on by default
    quit <- 0                #false implies go ahead and estimate
    
    while (iter == 0 || converge > tola) {   #MASTER CONTROL LOOP WHICH ITERATES UNTIL SOLUTION REACHED
      
      for (fb in 1:2) { #    MASTER fb LOOP       fb=1 is forward, 2 backward 
        dominate.out<- dominate(fb,issue,nperiods,nvar,mood,valid,smoothing,alpha, csign)  #master estimation routine
        alpha1<- dominate.out$alpha1
        alpha<- dominate.out$alpha
        mood[fb,]<- dominate.out$latent
      } #next fb
      
      fb <- 3 #average mood from here on
      for (p in 1:nperiods) { #    AVERAGE
        mood[fb, p] <- (mood[1, p] + mood[2, p]) / 2
      } #next p
      moodmean<-mean(mood[3,])
      sdmood<-sd(mood[3,])
      for (p in 1:nperiods) {  #PLACEMENT OF THIS LOOP MAY NOT BE RIGHT
        mood[fb,p] <- ((mood[fb,p] - moodmean) * ws / sdmood) + wm
      } #end for
      
      iter <- iter + 1 
      if (auto == "y") r<- iscorr(issue,mood) else auto <- "y"   #recompute correlations
      
      wtmean<- 0
      wtstd<- 0
      vsum<- 0
      goodvar<- 0
      converge<- 0 #start off default
      evalue<- 0
      totalvar<- 0
      
      for (v in 1:nvar) {
        wn<- nperiods-sum(is.na(issue[,v]))
        if (!is.na(sign(r[v]))) csign[v]<- sign(r[v])
        wn<- nperiods-sum(is.na(issue[,v]))
        if (wn>1) { #sum over variables actually used
          vratio <- wn / nperiods
          evalue <- evalue + vratio * r[v]^2
          totalvar <- totalvar + vratio
        } #end if
        evalue1 <- evalue
        totalvar1 <- totalvar
        #convergence tests
        if (wn > 3) {
          conv <- abs(r[v] - oldr[v])      #conv is convergence test for item=v
          conv <- conv * (wn / nperiods)                #weight criterion by number of available periods
          if (conv > converge)  converge <- conv        #converge is the global max of conv
        } #end if
        if (!is.na(r[v])) oldr[v] <- r[v]
        if (!is.na(r[v])) valid[v] <- r[v]^2
        if (!is.na(av[v])) wtmean <- wtmean + av[v] * valid[v]
        if (!is.na(std[v])) wtstd <- wtstd + std[v] * valid[v]
        if (!is.na(r[v])) vsum <- vsum + valid[v]
      } #end v loop
      
      if (vsum > 0)  wtmean <- wtmean / vsum
      if (vsum > 0)  wtstd <- wtstd / vsum
      if (pass == 1) {
        mean1 <- wtmean
        std1 <- wtstd
        e1=evalue
      } else {
        wtmean <- mean1
        wtstd <- std1 #*unexp
      } #end if
      fbcorr <- cor(mood[1,],mood[2,]) #fnfrontback 
      if (quit != 1) {
        cv<- format(round(converge,4),nsmall=4) 
        itfmt<-format(round(iter),justify="right",length=4)
        iter_hist <- rbind(iter_hist, data.frame(Dimension = paste0("Dimension ", pass), Iter = itfmt, Convergence = cv, Criterion = tola, 
                                                 Reliability = fbcorr, Alphaf = alpha1, Alphab = alpha))
      }
      if (converge > lastconv)  tola <- tola * 2
      lastconv <- converge
      
      auto = "y"  #skip corr on iter=1, set auto on
      
      if (iter >= 50) break #get out of while loop
    } #END MASTER WHILE ITERATION CONTROL LOOP
    
    if (auto == "y" && converge<tola) { #IF WE REACH THIS CODE WE HAVE A FINAL SOLUTION TO BE REPORTED
      if (pass == 1) out1<- out #hold output for 2 dimensional solution
      auto <- "Q"
      quit <- 1                #flag solution reached, last time through
      r<- iscorr(issue,mood)   #final iteration correlations
      if (pass == 1) r1<- r #hold correlations for 2 dimensional solution
      
      if (pass > 1) {
        unexp <- totalvar 
        totalvar <- unexp * totalvar
        evalue <- evalue * unexp
      } #    end if
      
      if (pass == 1) {
        expprop <- evalue / totalvar
        tot1 <- totalvar
        tot_var <- tot1
        expprop1 <- expprop
      } else {
        erel <- evalue / totalvar          #% exp relative
        totalvar <- (1 - expprop) * tot1   #true var=original var discounted by %exp
        evalue <- erel * totalvar          #rescale to retain %exp relationship
        expprop <- evalue / tot1           #now reduce eral to expprop
        evalue2 <- evalue
        expprop2 <- expprop
      } #    end if
      
      for (v in 1:nvar) {
        N[v]<- sum(!is.na(issue[,v]))
      }
      var.out<- list(varname=vl,loadings=r,means=av,std.deviations=std)
      if (pass !=  2 && npass>1) {
        for (v in 1:nvar) { 
          valid[v] <- 0               #reset all, regmoodissue will set good=1
          if (csign[v] != 0)  issue[,v]<- residmi(issue,v,mood)   #regmoodissue()
        } #v loop
      }  # if
      #begin prn output routine # mood[fb,] is now our estimate,    WHAT ABOUT A SECOND DIMENSION
      
      var_exp <- data.frame(Dimension =  "Dimension 1", 
                            Variance = evalue1, 
                            Proportion = expprop1)
      if(!is.null(evalue2)){
        var_exp <- rbind(var_exp, 
                         data.frame(Dimension = "Dimension 2", 
                                    Variance = evalue2, 
                                    Proportion = expprop2))
      }
      
      latent<- mood[fb,] #vector holds values for output
      if (pass == 1) latent1<- latent #hold first dimension
      #for Zelig output
      if (npass==1) {
        extract.out<- list(call=formula,
                           T=nperiods,
                           nvar=nvar,
                           unit=unit,
                           dimensions=npass,
                           period=period,
                           varname=vl,
                           N=N,
                           wtdmean=wtmean,
                           wtdstd=wtstd,
                           means=av,
                           std.deviations=std,
                           setup1=out1,
                           loadings1=r1,
                           latent1=latent1, 
                           hist = iter_hist, 
                           totalvar = totalvar1, 
                           var_exp = var_exp, 
                           dropped=dropped, 
                           smoothing = smoothing, 
                           ms_list = ms_list)
      } else {
        extract.out<- list(call=formula,
                           T=nperiods,
                           nvar=nvar,
                           unit=unit,
                           dimensions=npass,
                           period=period,
                           varname=vl,
                           N=N,
                           wtdmean=wtmean,
                           wtdstd=wtstd,
                           means=av,
                           std.deviations=std,
                           setup1=out1,
                           loadings1=r1,
                           latent1=latent1,
                           setup2=out,
                           loadings2=r,
                           latent2=latent, 
                           hist = iter_hist,
                           totalvar = totalvar1, 
                           var_exp = var_exp, 
                           dropped=dropped, 
                           smoothing = smoothing, 
                           ms_list = ms_list)
      }
    } #end if auto="y" 
  } #end of for pass=1,2 loop 
  
  # par(col=1) #reset on termination
  class(extract.out)<- "extract"
  return(extract.out)
}

#' Bootstrap the Dyad Ratios estimate
#' 
#' The bootstrapping algorithm was added to the package and code by Dave Armstrong. 
#' It is worth noting that this was not something Stimson had originally implemented
#' nor even something that he necessarily even endorsed.  While bootstrapping is a common
#' method for estimating uncertainty in complex estimators, like this one, use these
#' results at your own risk.  
#'
#' Generates a sampling distribution around the latent mood trajectory by
#' repeatedly drawing synthetic survey marginals from a binomial model and
#' re-running \code{\link{extract}}.  The original (unperturbed) estimate is
#' used as the point estimate; the bootstrap draws characterise uncertainty
#' around it.  
#'
#' All model parameters (aggregation interval, column names, smoothing, etc.)
#' are taken directly from the stored call inside \code{obj}, so there is no
#' risk of the bootstrap replications being run with different settings than
#' the original.
#'
#' @param obj   An object of class \code{"extract"} returned by
#'   \code{\link{extract}}.  The stored \code{call} is used to replay the
#'   estimation identically on each bootstrap draw.
#' @param data  The original data frame that was passed to \code{\link{extract}}
#'   to produce \code{obj}.  The index column is perturbed for each replication;
#'   all other columns are used as-is.
#' @param R         Integer. Number of bootstrap replications. Default \code{200}.
#' @param level     Numeric in (0, 1). Confidence level for the interval.
#'   Default \code{0.95}.
#' @param pw        Logical. Whether to calculate pairwise differences between times
#' @param seed      Integer or \code{NULL}. Passed to \code{set.seed} before
#'   the bootstrap loop. Default \code{NULL}.
#' @param parallel  Logical. Parallelise the outer loop using \pkg{parallel} if
#'   available? Useful for large \code{R}. Default \code{FALSE}.
#'
#' @return A list of class \code{"boot_dr"} with two or four components
#'   depending on whether the original \code{obj} used one or two dimensions:
#'   \describe{
#'     \item{\code{estimates}}{A \code{data.frame} with one row per time period
#'       and columns \code{period}, \code{year}, \code{month}, \code{quarter},
#'       \code{mood} (original point estimate), \code{lower}, and \code{upper}
#'       (confidence bounds at \code{(1-level)/2} and \code{1-(1-level)/2}).
#'       When \code{n_dim = 2}, three additional columns are appended:
#'       \code{mood_dim2}, \code{lower_dim2}, and \code{upper_dim2}.}
#'     \item{\code{samples}}{An \code{n_periods} × \code{R} matrix of raw
#'       bootstrap dimension-1 trajectories.}
#'     \item{\code{samples_dim2}}{(\code{n_dim = 2} only) An \code{n_periods}
#'       × \code{R} matrix of raw bootstrap dimension-2 trajectories.}
#'   }
#'   The list also carries attributes \code{R} (number of successful
#'   replications), \code{level}, \code{agg_interval}, and \code{n_dim}.
#'
#' @details
#' Each replication draws \eqn{y_i \sim \text{Binomial}(n_i, p_i)} where
#' \eqn{p_i} is the observed proportion and \eqn{n_i} is the sample size, then
#' replaces the index column with \eqn{y_i / n_i} (rescaled to the same 0–100
#' vs 0–1 convention as the original) and re-runs \code{\link{extract}} using
#' the exact call stored in \code{obj}.  Replications that error (e.g. due to
#' a degenerate draw) are silently discarded; \code{attr(result, "R")} reports
#' how many succeeded.
#'
#' @seealso \code{\link{extract}}, \code{\link{plot.boot_dr}}
#'
#' @examples
#' # Build a small synthetic dataset: 4 items measured annually over 20 years
#' set.seed(42)
#' n_years <- 20
#' years   <- seq(1980, length.out = n_years)
#' items   <- c("item_a", "item_b", "item_c", "item_d")
#'
#' dat <- do.call(rbind, lapply(items, function(item) {
#'   data.frame(
#'     varname = item,
#'     date    = as.Date(paste0(years, "-07-01")),
#'     index   = 50 + cumsum(rnorm(n_years, 0, 1.5)) + rnorm(n_years, 0, 2),
#'     n       = sample(800:1200, n_years, replace = TRUE)
#'   )
#' }))
#'
#' # Run the original estimate first
#' res <- extract(dat, n_col = "n", smoothing = FALSE)
#'
#' # Bootstrap with 100 replications (use more in practice)
#' boot <- boot_dr(res, dat, R = 100, seed = 1)
#'
#' # estimates is the summary data frame
#' head(boot$estimates)
#'
#' # mood is the original point estimate; lower/upper are the 95% CI
#' boot$estimates[, c("year", "mood", "lower", "upper")]
#'
#' # samples is the raw n_periods x R matrix of bootstrap trajectories
#' dim(boot$samples)
#'
#' # Plot the trajectory with uncertainty ribbon
#' plot(boot)
#'
#' @importFrom utils combn
#' @export
boot_dr <- function(obj, data, R = 200L,
                    level = 0.95,
                    pw = FALSE,
                    seed = NULL, parallel = FALSE) {

  stopifnot(inherits(obj, "extract"))
  stopifnot(is.data.frame(data))
  stopifnot(level > 0, level < 1)
  if (!is.null(seed)) set.seed(seed)

  # ---- recover column names and settings from the stored call ---------------
  call_list <- as.list(obj$call)
  index_col <- if (!is.null(call_list$index_col)) as.character(call_list$index_col) else "index"
  n_col     <- if (!is.null(call_list$n_col))     as.character(call_list$n_col)     else NULL
  n_dim     <- obj$settings$n_dim   # 1 or 2

  if (!index_col %in% names(data))
    stop("Column '", index_col, "' not found in data.")
  if (!is.null(n_col) && !n_col %in% names(data))
    stop("Column '", n_col, "' not found in data.")

  # ---- argument list for replaying the call ---------------------------------
  # as.list(call)[-1] drops the function name; we replace $data each iteration
  boot_args <- as.list(obj$call)[-1]

  # ---- prepare binomial draw parameters ------------------------------------
  idx <- data[[index_col]]
  ns  <- if (!is.null(n_col)) as.numeric(data[[n_col]]) else rep(1000, nrow(data))

  pct_scale <- max(idx, na.rm = TRUE) > 1
  prop <- if (pct_scale) idx / 100 else idx
  prop <- pmin(pmax(prop, 0), 1)

  one_rep <- function(r) {
    sim_y          <- rbinom(length(prop), size = round(ns), prob = prop)
    sim_index      <- sim_y / ns
    if (pct_scale) sim_index <- sim_index * 100

    d              <- data
    d[[index_col]] <- sim_index

    the_args       <- boot_args
    the_args$data  <- d

    res <- tryCatch(do.call(extract, the_args), error = function(e) NULL)
    if (is.null(res)) return(NULL)
    if (n_dim == 2L)
      list(mood = res$mood, mood_dim2 = res$mood_dim2)
    else
      res$mood
  }

  # ---- run replications -----------------------------------------------------
  if (parallel && requireNamespace("parallel", quietly = TRUE)) {
    nc <- max(1L, parallel::detectCores() - 1L)
    cl <- parallel::makeCluster(nc)
    on.exit(parallel::stopCluster(cl), add = TRUE)
    parallel::clusterExport(cl,
      c("data", "prop", "ns", "index_col", "pct_scale", "boot_args", "n_dim"),
      envir = environment())
    parallel::clusterEvalQ(cl, library(DyadRatios))
    results <- parallel::parLapply(cl, seq_len(R), one_rep)
  } else {
    results <- lapply(seq_len(R), one_rep)
  }

  ok      <- !sapply(results, is.null)
  results <- results[ok]
  if (length(results) == 0L) stop("All bootstrap replications failed.")

  # ---- build sample matrices ------------------------------------------------
  alpha <- (1 - level) / 2
  if (n_dim == 2L) {
    mood_mat      <- do.call(cbind, lapply(results, `[[`, "mood"))
    mood_mat_dim2 <- do.call(cbind, lapply(results, `[[`, "mood_dim2"))
  } else {
    mood_mat <- do.call(cbind, results)
  }

  # ---- assemble summary data frame ------------------------------------------
  estimates <- cbind(
    obj$periods,
    data.frame(
      mood  = obj$mood,
      lower = apply(mood_mat, 1, quantile, probs = alpha,     na.rm = TRUE),
      upper = apply(mood_mat, 1, quantile, probs = 1 - alpha, na.rm = TRUE),
      stringsAsFactors = FALSE
    )
  )
  if (n_dim == 2L) {
    estimates$mood_dim2  <- obj$mood_dim2
    estimates$lower_dim2 <- apply(mood_mat_dim2, 1, quantile, probs = alpha,     na.rm = TRUE)
    estimates$upper_dim2 <- apply(mood_mat_dim2, 1, quantile, probs = 1 - alpha, na.rm = TRUE)
  }

  # ---- assemble return list -------------------------------------------------
  out <- list(estimates = estimates, samples = mood_mat)
  if (n_dim == 2L) out$samples_dim2 <- mood_mat_dim2

  # ---- calculate optional pairwise differences ------------------------------
  if (pw) {
    period <- estimates$period
    combs  <- combn(nrow(mood_mat), 2)
    D      <- matrix(0, nrow = nrow(mood_mat), ncol = ncol(combs))
    D[cbind(combs[1, ], seq_len(ncol(combs)))] <- -1
    D[cbind(combs[2, ], seq_len(ncol(combs)))] <-  1
    diffs  <- t(mood_mat) %*% D
    p_diff <- apply(diffs, 2, function(x) mean(x > 0))
    p_diff <- ifelse(p_diff < 0.5, 1 - p_diff, p_diff)
    diff   <- matrix(estimates$mood, nrow = 1) %*% D
    out$pw <- data.frame(
      p1     = period[combs[1, ]],
      p2     = period[combs[2, ]],
      diff   = c(diff),
      p_diff = p_diff
    )
  }

  attr(out, "R")            <- sum(ok)
  attr(out, "level")        <- level
  attr(out, "agg_interval") <- obj$settings$agg_interval
  attr(out, "n_dim")        <- n_dim
  class(out) <- "boot_dr"
  out
}


#' Plot a boot_dr object
#'
#' Draws a ribbon plot of the bootstrap confidence interval around the
#' original mood estimate.  Requires \pkg{ggplot2}; falls back to base R if
#' unavailable.
#'
#' @param x     A \code{boot_dr} object returned by \code{\link{boot_dr}}.
#' @param dim   Integer.  Which dimension to plot (\code{1} or \code{2}).
#'   Dimension 2 is only available when the original \code{\link{extract}}
#'   call used \code{n_dim = 2}.  Default \code{1}.
#' @param title Character. Plot title.
#' @param ylab  Character. Y-axis label.
#' @param ...   Ignored.
#' @return Invisibly, the ggplot2 object or \code{NULL} (base R).
#' @importFrom graphics polygon lines
#' @importFrom grDevices adjustcolor
#' @export
#' @method plot boot_dr
plot.boot_dr <- function(x, dim = 1L,
                         title = "Estimated Public Mood",
                         ylab = "Mood", ...) {
  est <- x$estimates
  n_dim_boot <- attr(x, "n_dim")

  if (dim == 2L) {
    if (is.null(n_dim_boot) || n_dim_boot < 2L ||
        !all(c("mood_dim2", "lower_dim2", "upper_dim2") %in% names(est)))
      stop("Dimension 2 results are not available in this boot_dr object.")
    mood_col  <- "mood_dim2"
    lower_col <- "lower_dim2"
    upper_col <- "upper_dim2"
  } else {
    mood_col  <- "mood"
    lower_col <- "lower"
    upper_col <- "upper"
  }

  date_vec <- as.Date(paste(est$year,
                            ifelse(est$month > 0, est$month, 1),
                            "01", sep = "-"))

  if (requireNamespace("ggplot2", quietly = TRUE)) {
    df <- data.frame(date  = date_vec,
                     mood  = est[[mood_col]],
                     lower = est[[lower_col]],
                     upper = est[[upper_col]])
    level_pct <- round(attr(x, "level") * 100)
    dim_label <- if (dim == 2L) "  (dimension 2)" else ""
    p <- ggplot2::ggplot(df, ggplot2::aes(x = date)) +
      ggplot2::geom_ribbon(ggplot2::aes(ymin = lower, ymax = upper),
                           fill = "#2c7bb6", alpha = 0.2) +
      ggplot2::geom_line(ggplot2::aes(y = mood),
                         colour = "#2c7bb6", linewidth = 0.8) +
      ggplot2::labs(title    = paste0(title, dim_label),
                    subtitle = sprintf("%d%% bootstrap confidence interval  (R = %d)",
                                       level_pct, attr(x, "R")),
                    x = "Year", y = ylab) +
      ggplot2::theme_minimal(base_size = 13) +
      ggplot2::theme(plot.title = ggplot2::element_text(face = "bold"))
    print(p)
    return(invisible(p))
  }

  # Base R fallback
  plot(date_vec, est[[mood_col]], type = "l", col = "#2c7bb6",
       ylim = range(c(est[[lower_col]], est[[upper_col]]), na.rm = TRUE),
       main = title, xlab = "Year", ylab = ylab, ...)
  polygon(c(date_vec, rev(date_vec)),
          c(est[[lower_col]], rev(est[[upper_col]])),
          col = adjustcolor("#2c7bb6", alpha.f = 0.2), border = NA)
  lines(date_vec, est[[mood_col]], col = "#2c7bb6")
  invisible(NULL)
}


#' Print method for extract objects
#'
#' @param x A \code{extract} object returned by \code{\link{extract}}.
#' @param ... Ignored.
#' @export
print.extract <- function(x, ...) {
  cat("\n=== Dyad Ratios Estimation ===\n\n")
  s <- x$settings
  cat(sprintf("Aggregation:   %s\n", s$agg_interval))
  cat(sprintf("Period:        %s to %s\n",
              format(s$start_date, "%Y-%m-%d"),
              format(s$end_date,   "%Y-%m-%d")))
  cat(sprintf("Time points:   %d\n", x$n_periods))
  cat(sprintf("Series:        %d\n", x$n_series))
  cat(sprintf("Observations:  %d\n", x$n_obs))
  cat(sprintf("Dimensions:    %d\n", s$n_dim))
  cat(sprintf("Smoothing:     %s\n", if (s$smoothing) "on" else "off"))
  if (s$smoothing) {
    cat(sprintf("  Alpha (F):   %.3f\n", x$alpha_F))
    cat(sprintf("  Alpha (B):   %.3f\n", x$alpha_B))
  }
  cat(sprintf("Iterations:    %d\n",
              max(x$iterations$iter, na.rm = TRUE)))
  cat(sprintf("Eigenvalue:    %.3f\n", x$eigenvalue))
  cat(sprintf("Var. Explained:%.1f%%\n\n",
              100 * x$variance_explained))

  cat("Mood (first 10 periods):\n")
  idx <- seq_len(min(10, x$n_periods))
  pd  <- x$periods[idx, , drop = FALSE]
  agg <- x$settings$agg_interval
  labels <- if (agg %in% c("quarterly")) {
    paste0(pd$year, " Q", pd$quarter)
  } else if (agg == "monthly") {
    paste0(pd$year, "-", sprintf("%02d", pd$month))
  } else if (agg == "daily") {
    paste0(pd$year, "-", sprintf("%02d", pd$month))
  } else {
    as.character(pd$year)
  }
  vals <- round(x$mood[idx], 2)
  cat(paste(sprintf("  %-10s %.2f", labels, vals), collapse = "\n"))
  if (x$n_periods > 10) cat(sprintf("\n  ... (%d more periods)", x$n_periods - 10))
  cat("\n\n")
  invisible(x)
}


#' Summary method for extract objects
#'
#' Prints a detailed report similar to the log file produced by the original
#' 'MCalc' program, including the iteration history, variable loadings, and
#' variance accounting.
#'
#' @param object A \code{extract} object.
#' @param ... Ignored.
#' @return Invisibly returns \code{object}.
#' @method summary extract
#' @export
summary.extract <- function(object, ...) {
  x <- object
  s <- x$settings
  cat("\n================================================================\n")
  cat("Dyad Ratios Estimation Report\n")
  cat("================================================================\n\n")
  cat(sprintf("Aggregation interval: %s\n", s$agg_interval))
  cat(sprintf("Period: %s to %s\n",
              format(s$start_date, "%Y-%m-%d"),
              format(s$end_date,   "%Y-%m-%d")))
  cat(sprintf("Time points:  %d\n", x$n_periods))
  cat(sprintf("Series used:  %d\n", x$n_series))
  cat(sprintf("Total observations: %d\n", x$n_obs))
  cat(sprintf("Dimensions extracted: %d\n", s$n_dim))
  cat(sprintf("Exponential smoothing: %s\n",
              if (s$smoothing) "on" else "off"))
  cat(sprintf("Convergence tolerance: %.4f\n\n", s$tol))

  cat("----------------------------------------------------------------\n")
  cat("Iteration History\n")
  cat("----------------------------------------------------------------\n")
  cat(sprintf("%-5s  %-11s  %-8s  %-11s  %-8s  %-8s\n",
              "Iter", "Converge", "Crit", "Reliability", "Alpha_F", "Alpha_B"))
  cat(strrep("-", 60), "\n")
  for (i in seq_len(nrow(x$iterations))) {
    r <- x$iterations[i, ]
    cat(sprintf("%5d  %11.5f  %8.3f  %11.3f  %8.3f  %8.3f\n",
                r$iter, r$converge, r$criterion,
                r$reliability, r$alpha_F, r$alpha_B))
  }

  cat("\n----------------------------------------------------------------\n")
  cat("Variable Loadings\n")
  cat("----------------------------------------------------------------\n")
  lo <- x$loadings
  has_dim2 <- s$n_dim == 2 && "dim2" %in% names(lo)
  hdr <- sprintf("%-20s  %5s  %8s  %8s  %8s",
                 "Variable", "N", "Mean", "Std", "Dim1")
  if (has_dim2) hdr <- paste0(hdr, "     Dim2")
  cat(hdr, "\n", strrep("-", if (has_dim2) 66 else 55), "\n", sep = "")
  for (i in seq_len(nrow(lo))) {
    ln <- sprintf("%-20s  %5d  %8.3f  %8.3f  %8.3f",
                  lo$variable[i], lo$n_obs[i],
                  lo$mean[i], lo$std_dev[i], lo$dim1[i])
    if (has_dim2) ln <- paste0(ln, sprintf("  %8.3f", lo$dim2[i]))
    cat(ln, "\n")
  }

  cat("\n----------------------------------------------------------------\n")
  cat("Variance Accounting\n")
  cat("----------------------------------------------------------------\n")
  cat(sprintf("Eigenvalue estimate:    %.3f\n", x$eigenvalue))
  cat(sprintf("Variance explained:     %.1f%%", 100 * x$variance_explained))
  if (s$n_dim == 2 && !is.na(x$variance_explained_dim2)) {
    cat(sprintf("  (dim 1)\n"))
    cat(sprintf("Variance explained:     %.1f%%  (dim 2)\n",
                100 * x$variance_explained_dim2))
    cat(sprintf("Variance explained:     %.1f%%  (combined)\n",
                100 * (x$variance_explained + x$variance_explained_dim2)))
  } else {
    cat("\n")
  }
  cat("\n")

  invisible(x)
}

#' Plot estimated public mood
#'
#' Produces a time-series plot of the estimated latent mood.  Requires
#' \pkg{ggplot2}.  If \pkg{ggplot2} is not installed, falls back to base R
#' graphics.
#'
#' @param x A \code{extract} object.
#' @param dim Integer.  Which dimension to plot (1 or 2).  Default \code{1}.
#' @param title Character.  Plot title.  Default \code{"Estimated Public Mood"}.
#' @param ylab Character.  Y-axis label.  Default \code{"Mood"}.
#' @param ... Additional arguments passed to the plotting function.
#' @return Invisibly, the plot object (ggplot2) or \code{NULL} (base R).
#' @importFrom graphics abline plot
#' @importFrom stats sd
#' @export
#' @method plot extract
plot.extract <- function(x, dim = 1L, title = "Estimated Public Mood",
                    ylab = "Mood", ...) {
  stopifnot(inherits(x, "extract"))
  mood_vec <- if (dim == 2 && x$settings$n_dim == 2) x$mood_dim2 else x$mood
  pd <- x$periods

  # Build a date approximation from the period table
  date_vec <- as.Date(paste(pd$year,
                            ifelse(pd$month > 0, pd$month, 1),
                            "01", sep = "-"))

  if (requireNamespace("ggplot2", quietly = TRUE)) {
    df <- data.frame(date = date_vec, mood = mood_vec)
    p <- ggplot2::ggplot(df, ggplot2::aes(x = date, y = mood)) +
      ggplot2::geom_line(colour = "#2c7bb6", linewidth = 0.8) +
      ggplot2::geom_hline(yintercept = mean(mood_vec, na.rm = TRUE),
                          linetype = "dashed", colour = "grey50") +
      ggplot2::labs(title = title,
                    x = "Year",
                    y = ylab) +
      ggplot2::theme_minimal(base_size = 13) +
      ggplot2::theme(plot.title = ggplot2::element_text(face = "bold"))
    print(p)
    return(invisible(p))
  } else {
    plot(date_vec, mood_vec, type = "l", col = "#2c7bb6",
         main = title, xlab = "Year", ylab = ylab, ...)
    abline(h = mean(mood_vec, na.rm = TRUE), lty = 2, col = "grey50")
    return(invisible(NULL))
  }
}

#' Print method for extract function
#' 
#' @param x An object of class "extract", typically a result of the extract function.
#' @param ... Other arguments to be pased down
#' @return A printed summary of the output from the extract function and the input object returned invisbly. 
#' @usage NULL
#' 
#' @method print extract
#' @export
#' @examples
#' data(jennings)
#' dr_out <- extract(varname = jennings$variable, 
#'                   date = jennings$date, 
#'                   index = jennings$value, 
#'                   ncases = jennings$n, 
#'                   begindt = min(jennings$date), 
#'                   enddt = max(jennings$date), 
#'                   npass=1)
#' print(dr_out)
print.extract <- function(x, ...) {
  cat("\nEstimation report:\n")
  cat("Period:", min(x$period), " to", max(x$period), ", ", x$T, " time points\n")
  cat("Number of series: ", x$nvar + ifelse(is.null(x$dropped), 0, nrow(x$dropped)), "\n")
  cat("Number of usable series: ", x$nvar, "\n")
  cat("Exponential smoothing: ", ifelse(x$smoothing, "Yes", "No"), "\n\n")
  cat("Iteration history: \n")
  cat(" \n")
  print(x$hist, row.names = FALSE, digits=3)
  cat(" \n")
  cat("Total Variance to be Explained = ", sprintf("%.2f", x$totalvar), "\n\n")
  cat("Percent Variance Explained: \n")
  print(x$var_exp, row.names=FALSE, digits=3)
  cat(" \n")
  cat("Final Weighted Average Metric: ", "Mean: ", sprintf("%.2f", x$wtdmean), 
      " St. Dev: ", sprintf("%.2f", x$wtdstd), "\n")
  invisible(x)
}

#' Create matrix of issue data aggregated by period. This is a low-level function called by extract(), not intended to be called directly by the user
#' @noRd
aggregate <-
function(varname,date,index,ncases,mindate,maxdate,nperiods,nvar,aggratio,unit,miny,minper) {   #
    #READ A NEW RECORD, CALCULATE PERIOD, AND SET UP AGGREGATION INTO MAT.ISSUE[NPERIODS,NVAR] 
  vl<- character(nvar)
  mind<- as.integer(mindate)
  maxd<- as.integer(maxdate)
  vfac<- factor(varname) #make a factor vector
  vlev<- levels(vfac)    #find unique categories
  Mat.Issue<- array(dim=c(nperiods,nvar))

  nrec<-length(varname) #added for R compatibility
  lp<- 0
  per<- 0
  x<- 0
  c<- 0
  nkeep<- 0
  lv<- "0"
  for (record in 1:nrec) { # MASTER LOOP THROUGH INPUT DATA, 1 TO NREC
    if (ncases[record] == 0 || is.na(ncases[record])) ncases[record] <- 1000
    mo <- month(date[record])
    qu <- 1 + as.integer((mo - 1)/3)
    dy <- day(date[record])
    yr <- year(date[record])
    curdate<- as.integer(date[record])
    if (curdate >= mind &&  curdate <= maxd) {  #is date within range?
    nkeep <- nkeep + 1
    if (nkeep==1) { #startup routine for first good case
      firstcase<- TRUE
      lp <- findper(unit,curdate,mind,miny,minper,aggratio)
      lv <- varname[record]
      x <- index[record] * ncases[record] #start new sums for case 1
      c <- ncases[record]
      for (i in 1:nvar) {
        if (lv==vlev[i]) v=i #determine v by matching to position of labels vector
        } #end for
      } else {
      firstcase<- FALSE
      } #end if
    if (firstcase == FALSE) { #skip over the rest for first good case
    per<- findper(unit,curdate,mind,miny,minper,aggratio) #here we translate date into agg category
    if ((varname[record] !=  lv) || (per !=lp)) { #found a new period or variable name
      if (lp > 0 &&  lp <= nperiods) {
        Mat.Issue[lp, v] <- x / c #recompute for either period or var change
        x<- 0
        c<- 0
        }
      if (varname[record] !=  lv) { #new var only
        for (i in 1:nvar) {
          if (varname[record]==vlev[i]) v=i #determine v by matching to position of labels vector
          } #end for
        vl[v]<- varname[record] #this will only catch names that have good cases
        lv<-vl[v]  #reassign new varname to lastvar
      } # new var
      lp <- findper(unit,curdate,mind,miny,minper,aggratio)
      x <- index[record] * ncases[record] #start new sums for current case
      c <- ncases[record]
    } else {
      x<- x + index[record] * ncases[record] #a continuing case, increment sums
      c<- c + ncases[record]
    }
    } # end of first case special loop
  } #end of date test loop
  } #newrec: next record
  vl<- vlev #overwrite previous assignment which had good names only
  agglist<- list(lab=vl,iss=Mat.Issue)
  return(agglist) #list includes labels and issue matrix
  }

# 
# mindt <- min(mindate)
# maxdt <- max(maxdate)
# minyr <- lubridate::year(mindt)
# maxyr <- lubridate::year(maxdt)
# minmo <- lubridate::month(mindt)
# maxmo <- lubridate::month(maxdt)
# minday <- lubridate::day(mindt)
# maxday <- lubridate::day(maxdt)
# minq <- lubridate::quarter(mindt)
# maxq <- lubridate::quarter(maxdt)
#   
# if(unit == "A"){
#   periods <- seq(minyr, maxyr, by=1)
#   obs_periods <- lubridate::year(date)
# }
# if(unit == "Q"){
#   stq <- lubridate::yq(paste0(minyr,"-",minq))
#   endq <- lubridate::yq(paste0(maxyr,"-",maxq))
#   periods <- seq(stq, endq, by="quarter")
#   obs_periods <- lubridate::yq(paste0(lubridate::year(date),"-",lubridate::quarter(date)))
# }
# if(unit == "M"){
#   stm <- lubridate::ym(paste0(minyr,"-",minmo))
#   endm <- lubridate::ym(paste0(maxyr,"-",maxmo))
#   periods <- seq(stm, endm, by="month")
#   obs_periods <- lubridate::ym(paste0(lubridate::year(date),"-",lubridate::month(date)))
# }
# if(unit == "D"){
#   periods <- seq(mindt, maxdt, by="day")
#   obs_periods <- date
# }
# if(unit == "O"){
#   year_periods <- seq(lubridate::ymd(paste0(minyr, "-01-01")), 
#                       lubridate::ymd(paste0(maxyr, "-01-01")), 
#                       by=paste0(mult, " year"))
#   periods <- lubridate::year(year_periods)
#   obs_years <- lubridate::year(date)
#   if(any(obs_years > max(periods)))periods <- c(periods, max(periods) + mult)
#   nper <- length(periods)
#   obs_period <- rep(NA, length(date))
#   o <- outer(date, periods, \(x,y){
#     lubridate::year(x) >= y & lubridate::year(x) < (y + mult)
#   })
#   obs_period <- periods[apply(o, 1, which)]
# }
# 
# index <- input$value
# if(any(index > 1)) index <- index/100
# 
# all_dat <- data.frame(period = periods)
# ag_dat <- data.frame(period = obs_period, 
#                      var = varname, 
#                      index = index, 
#                      ncases = ncases)
# 
# ag_dat <- ag_dat |> 
#   dplyr::mutate(n_pos = index * ncases) |> 
#   dplyr::group_by(period, var) |> 
#   dplyr::summarise(n_pos = sum(n_pos), 
#             ncases = sum(ncases)) |> 
#   dplyr::mutate(pct = n_pos / ncases) |> 
#   dplyr::select(period, var, pct) |> 
#   tidyr::pivot_wider(names_from = var, values_from = pct) 
# 
# Mat.Issue <- ag_dat |> dplyr::ungroup() |> dplyr::select(-period) |> as.matrix()

#' Extract mood estimates as a data frame
#'
#' Combines the period descriptor table from an \code{\link{extract}} result
#' with the estimated mood trajectory, returning a plain \code{data.frame}
#' that is convenient for further analysis or export.
#'
#' @param obj An object of class \code{"extract"} returned by
#'   \code{\link{extract}}.
#' @param ... Ignored; included for potential future use.
#'
#' @return A \code{data.frame} with one row per time period.  Columns
#'   \code{period}, \code{year}, \code{month}, and \code{quarter} are
#'   inherited from the period table inside \code{obj}.  When a single
#'   dimension was estimated an additional column \code{mood} is appended.
#'   When two dimensions were estimated, columns \code{mood_dim1} and
#'   \code{mood_dim2} are appended instead.
#'
#' @examples
#' set.seed(1)
#' dat <- data.frame(
#'   varname = rep(c("a", "b", "c"), each = 20),
#'   date    = rep(seq(as.Date("1980-01-01"), by = "year", length.out = 20), 3),
#'   index   = 50 + rnorm(60, 0, 5),
#'   n       = 1000L
#' )
#' res <- extract(dat, n_col = "n", smoothing = FALSE)
#' mood_df <- get_mood(res)
#' head(mood_df)
#'
#' @export
get_mood <- function(obj, ...) {
  out <- obj$periods
  if (is.na(obj$mood_dim2[1])) {
    out$mood     <- obj$mood
  } else {
    out$mood_dim1 <- obj$mood
    out$mood_dim2 <- obj$mood_dim2
  }
  out
}

#' Run the Dyad Ratios Algorithm
#'
#' Estimates one or two latent opinion dimensions from a collection of
#' time-series issue variables using the Dyad Ratios algorithm developed by
#' James Stimson (Stimson 1991, 1999).  This function accepts already-loaded
#' data as a \code{data.frame}, handles aggregation to the requested interval,
#' standardises the issue matrix, passes it to the compiled C++ core, and
#' returns a richly-annotated result object.
#'
#' @param data A \code{data.frame} with at minimum the columns specified in
#'   \code{varname_col}, \code{date_col}, \code{index_col}, and optionally
#'   \code{n_col}.  Each row represents one survey marginal: the proportion
#'   (or count) of respondents taking a particular position on one issue at
#'   one point in time.  The variable name column identifies which opinion
#'   item the row belongs to.
#' @param varname_col Character.  Name of the column that identifies the
#'   opinion variable (issue series).  Default \code{"varname"}.
#' @param date_col Character.  Name of the column containing the observation
#'   date.  Must be coercible to \code{Date} via \code{as.Date()}.
#'   Default \code{"date"}.
#' @param index_col Character.  Name of the column containing the survey
#'   marginal value (e.g. proportion liberal, per cent approving, etc.).
#'   Default \code{"index"}.
#' @param n_col Character or \code{NULL}.  Name of the column containing the
#'   number of respondents (sample size / weight).  When \code{NULL} (default)
#'   all observations receive equal weight of 1000, matching the original
#'   program's default.
#' @param agg_interval Character.  Temporal aggregation level.  One of
#'   \code{"annual"} (default), \code{"quarterly"}, \code{"monthly"},
#'   \code{"daily"}, or \code{"multi_year"}.
#' @param multiple Integer.  Number of years per period when
#'   \code{agg_interval = "multi_year"}.  Default \code{1}.
#' @param start_date Optional \code{Date} (or character coercible by
#'   \code{as.Date}) giving the earliest date to include.  Default: the
#'   earliest date found in \code{data}.
#' @param end_date Optional \code{Date} (or character coercible by
#'   \code{as.Date}) giving the latest date to include.  Default: the
#'   latest date found in \code{data}.
#' @param n_dim Integer 1 or 2.  Number of latent dimensions to extract.
#'   Default \code{1}.
#' @param smoothing Logical.  Apply optimal exponential smoothing to each
#'   forward and backward pass.  Default \code{TRUE}.
#' @param tol Numeric.  Convergence tolerance (maximum weighted change in
#'   item-mood correlations between iterations).  Default \code{0.001}.
#' @param fiscal_year_end Integer 1–12.  Final month of the fiscal / policy
#'   year; use 12 (default) for calendar years.  When set to a value < 12
#'   observations after this month are rolled forward into the next year,
#'   consistent with the original program's \code{FinalMonth} parameter.
#' @param smooth_seed Integer.  Seed for the C random number generator used
#'   inside the exponential-smoothing optimiser (\code{eSmooth}).  The default
#'   value of \code{1} replicates MCalc's implicit behaviour: MCalc never
#'   calls \code{srand()}, so C's \code{rand()} starts from seed 1 on every
#'   run.  Change this only if you need to explore sensitivity to the smoothing
#'   optimiser's random fallback step.  Has no effect when
#'   \code{smoothing = FALSE}.
#'
#' @return An object of class \code{"extract"}, which is a named list
#'   containing:
#'   \describe{
#'     \item{\code{mood}}{Numeric vector of estimated public mood, one value
#'       per period.  When \code{n_dim = 2} this is the first dimension.}
#'     \item{\code{mood_dim2}}{Numeric vector of the second dimension, or
#'       \code{NA} when \code{n_dim = 1}.}
#'     \item{\code{periods}}{A \code{data.frame} describing each period
#'       (year, month/quarter where applicable) matching the length of
#'       \code{mood}.}
#'     \item{\code{loadings}}{A \code{data.frame} of variable names with
#'       loading on each extracted dimension.}
#'     \item{\code{iterations}}{A \code{data.frame} of the iteration history
#'       (convergence, reliability, smoothing alphas per iteration).}
#'     \item{\code{n_series}}{Integer.  Number of opinion series retained
#'       after dropping constant series.}
#'     \item{\code{n_periods}}{Integer.  Number of time periods.}
#'     \item{\code{n_obs}}{Integer.  Total number of input observations used.}
#'     \item{\code{alpha_F}}{Final forward-pass smoothing parameter.}
#'     \item{\code{alpha_B}}{Final backward-pass smoothing parameter.}
#'     \item{\code{eigenvalue}}{Eigenvalue estimate for the first dimension.}
#'     \item{\code{variance_explained}}{Proportion of total variance accounted
#'       for by the first dimension.}
#'     \item{\code{variance_explained_dim2}}{(\code{n_dim = 2} only) Additional
#'       proportion of total variance accounted for by the second dimension (i.e.
#'       its share of the variance left unexplained by dimension 1).  \code{NA}
#'       for single-dimension solutions.}
#'     \item{\code{call}}{The matched \code{\link[base]{call}}.}
#'     \item{\code{settings}}{List of all parameter values used.}
#'   }
#'
#' @details
#' The algorithm iterates between a forward pass and a backward pass.  In each
#' pass every period's latent score is estimated as the weighted average of
#' ratios of issue values relative to all other periods for which both values
#' are non-missing.  The weights are the squared correlations of each issue
#' with the current mood estimate.  Convergence is declared when the maximum
#' weighted change in these correlations falls below \code{tol}.
#'
#' Issues are standardised to a mean of 100 and a standard deviation of 10
#' before estimation, then the final mood series is rescaled to have the
#' weighted mean and standard deviation of the raw issue series.
#'
#' @references
#' Stimson, J. A. (1991). \emph{Public Opinion in America: Moods, Cycles, and
#' Swings}. Westview Press.
#'
#' Stimson, J. A. (1999). \emph{Public Opinion in America}, 2nd ed.
#' Westview Press.
#'
#' @examples
#' # Minimal synthetic example
#' set.seed(42)
#' n <- 60
#' dates <- seq(as.Date("1980-01-01"), by = "year", length.out = n / 3)
#' dat <- data.frame(
#'   varname = rep(c("item_a", "item_b", "item_c"), each = length(dates)),
#'   date    = rep(dates, 3),
#'   index   = c(seq(40, 65, length.out = length(dates)) + rnorm(length(dates), 0, 2),
#'               seq(55, 35, length.out = length(dates)) + rnorm(length(dates), 0, 2),
#'               seq(45, 70, length.out = length(dates)) + rnorm(length(dates), 0, 3)),
#'   n       = 1000
#' )
#' result <- extract(dat, n_col = "n", smoothing = FALSE)
#' print(result)
#'
#' @export
extract <- function(data,
                        varname_col    = "varname",
                        date_col       = "date",
                        index_col      = "index",
                        n_col          = NULL,
                        agg_interval   = c("annual", "quarterly", "monthly",
                                           "daily", "multi_year"),
                        multiple       = 1L,
                        start_date     = NULL,
                        end_date       = NULL,
                        n_dim          = 1L,
                        smoothing      = TRUE,
                        tol            = 0.001,
                        fiscal_year_end = 12L,
                        smooth_seed    = 1L) {

  cl <- match.call()
  agg_interval <- match.arg(agg_interval)

  # ---- input validation ---------------------------------------------------
  stopifnot(is.data.frame(data))
  for (col in c(varname_col, date_col, index_col))
    if (!col %in% names(data))
      stop(sprintf("Column '%s' not found in data.", col))
  if (!is.null(n_col) && !n_col %in% names(data))
    stop(sprintf("Column '%s' (n_col) not found in data.", n_col))

  n_dim  <- as.integer(n_dim)
  stopifnot(n_dim %in% 1:2)
  multiple <- as.integer(multiple)
  stopifnot(multiple >= 1)
  fiscal_year_end <- as.integer(fiscal_year_end)
  stopifnot(fiscal_year_end >= 1 && fiscal_year_end <= 12)

  # ---- parse and filter dates --------------------------------------------
  data[[date_col]] <- as.Date(data[[date_col]])
  if (!is.null(start_date)) start_date <- as.Date(start_date)
  if (!is.null(end_date))   end_date   <- as.Date(end_date)

  min_date <- if (is.null(start_date)) min(data[[date_col]], na.rm = TRUE) else start_date
  max_date <- if (is.null(end_date))   max(data[[date_col]], na.rm = TRUE) else end_date

  data <- data[!is.na(data[[date_col]]) &
               data[[date_col]] >= min_date &
               data[[date_col]] <= max_date, , drop = FALSE]

  if (nrow(data) == 0)
    stop("No data remain after date filtering.")

  # ---- n (sample size / weight) ------------------------------------------
  if (is.null(n_col)) {
    data$.n_weight <- 1000L
  } else {
    data$.n_weight <- as.numeric(data[[n_col]])
    data$.n_weight[is.na(data$.n_weight) | data$.n_weight == 0] <- 1000L
  }

  # ---- fiscal year adjustment (matches original FinalMonth logic) ----------
  data$.year  <- as.integer(format(data[[date_col]], "%Y"))
  data$.month <- as.integer(format(data[[date_col]], "%m"))
  data$.day   <- as.integer(format(data[[date_col]], "%d"))

  if (fiscal_year_end < 12) {
    roll <- data$.month > fiscal_year_end
    data$.year[roll]  <- data$.year[roll] + 1L
    data$.month[roll] <- 1L
  }

  # ---- derive period index -----------------------------------------------
  data$.period <- .assign_period(data$.year, data$.month, data$.day,
                                 data[[date_col]],
                                 agg_interval, multiple,
                                 min_date, max_date)

  # ---- aggregate to period × variable matrix -----------------------------
  agg <- .aggregate_issues(
    varname      = as.character(data[[varname_col]]),
    index        = as.numeric(data[[index_col]]),
    n            = data$.n_weight,
    period       = data$.period,
    year         = data$.year,
    month        = data$.month,
    agg_interval = agg_interval,
    min_date     = min_date,
    multiple     = multiple
  )

  pIssue_raw <- agg$mat   # nperiods × nVar, 0 = NA sentinel
  var_names  <- agg$var_names
  period_df  <- agg$period_df
  nperiods   <- nrow(pIssue_raw)
  nVar       <- ncol(pIssue_raw)

  if (nVar < 2)
    stop("At least 2 non-constant opinion series are required.")

  # ---- drop constant (zero-variance) series -------------------------------
  keep <- apply(pIssue_raw, 2, function(x) {
    good <- x[x != 0]
    length(good) > 1 && sd(good) > 1e-4
  })
  if (sum(keep) < 2)
    stop("Fewer than 2 non-constant series remain after filtering.")
  pIssue_raw <- pIssue_raw[, keep, drop = FALSE]
  var_names  <- var_names[keep]
  nVar       <- ncol(pIssue_raw)

  # ---- standardise: 100 + 10 * z  (matching original code) ---------------
  # MCalc stores per-variable means as float (32-bit) via av[v]=float(s/nGood),
  # even though av[] is declared double.  Simulate that precision loss so that
  # the standardised values fed to the C++ core match MCalc exactly.
  to_float32 <- function(x)
    readBin(writeBin(as.double(x), raw(), size = 4L),
            double(), n = length(x), size = 4L)

  av_vec  <- numeric(nVar)
  std_vec <- numeric(nVar)
  for (v in seq_len(nVar)) {
    good <- pIssue_raw[, v]; good <- good[good != 0]
    av_vec[v]  <- mean(good)
    std_vec[v] <- sd(good)
  }
  av_vec <- to_float32(av_vec)   # match MCalc's float() cast on the mean

  for (v in seq_len(nVar)) {
    non_na <- pIssue_raw[, v] != 0
    pIssue_raw[non_na, v] <-
      100 + 10 * (pIssue_raw[non_na, v] - av_vec[v]) / std_vec[v]
  }

  # ---- call the C++ estimation core --------------------------------------
  cpp_out <- dyad_ratios_cpp(
    data_matrix = pIssue_raw,
    nperiods    = nperiods,
    nVar        = nVar,
    npass       = n_dim,
    smoothing   = smoothing,
    tola_in     = tol,
    beginper    = 1L,
    endper      = nperiods,
    raw_av      = av_vec,
    raw_std     = std_vec,
    seed        = as.integer(smooth_seed)
  )

  # ---- assemble loadings data frame --------------------------------------
  loadings_df <- data.frame(
    variable = var_names,
    mean     = av_vec,
    std_dev  = std_vec,
    n_obs    = colSums(pIssue_raw != 0),
    dim1     = cpp_out$loadings$dim1,
    stringsAsFactors = FALSE
  )
  if (n_dim == 2)
    loadings_df$dim2 <- cpp_out$loadings$dim2
  row.names(loadings_df) <- NULL

  # ---- build result -------------------------------------------------------
  result <- list(
    mood               = cpp_out$mood,
    mood_dim2          = if (n_dim == 2) cpp_out$mood_dim2 else NA_real_,
    periods            = period_df,
    loadings           = loadings_df,
    iterations         = cpp_out$iterations,
    n_series           = nVar,
    n_periods          = nperiods,
    n_obs              = nrow(data),
    alpha_F            = cpp_out$alpha_F,
    alpha_B            = cpp_out$alpha_B,
    eigenvalue         = cpp_out$eigenvalue,
    variance_explained      = cpp_out$variance_explained,
    variance_explained_dim2 = cpp_out$variance_explained_dim2,
    call               = cl,
    settings           = list(
      agg_interval    = agg_interval,
      multiple        = multiple,
      n_dim           = n_dim,
      smoothing       = smoothing,
      tol             = tol,
      start_date      = min_date,
      end_date        = max_date,
      fiscal_year_end = fiscal_year_end,
      smooth_seed     = as.integer(smooth_seed)
    )
  )
  class(result) <- "extract"
  result
}

# ============================================================================
# Internal helpers
# ============================================================================

#' Assign a period index to each observation
#' @noRd
.assign_period <- function(year, month, day, date_vec,
                            agg_interval, multiple,
                            min_date, max_date) {
  min_year  <- as.integer(format(min_date, "%Y"))
  min_month <- as.integer(format(min_date, "%m"))
  min_qtr   <- ceiling(min_month / 3)

  switch(agg_interval,
    annual = {
      as.integer((year - min_year) / multiple) + 1L
    },
    multi_year = {
      as.integer((year - min_year) / multiple) + 1L
    },
    quarterly = {
      qtr <- ceiling(month / 3)
      (year - min_year) * 4L - (min_qtr - 1L) + qtr
    },
    monthly = {
      (year - min_year) * 12L - (min_month - 1L) + month
    },
    daily = {
      as.integer(date_vec - min_date) + 1L
    }
  )
}

#' Weighted aggregate of issue values into period × variable matrix
#' Returns 0 (NA sentinel) for missing cells.
#' @noRd
.aggregate_issues <- function(varname, index, n, period, year, month,
                               agg_interval, min_date, multiple = 1L) {
  vars      <- sort(unique(varname))
  nVar      <- length(vars)
  max_period <- max(period, na.rm = TRUE)
  min_period <- min(period, na.rm = TRUE)
  nperiods  <- max_period - min_period + 1L

  mat <- matrix(0.0, nrow = nperiods, ncol = nVar)
  colnames(mat) <- vars

  for (i in seq_along(varname)) {
    p  <- period[i] - min_period + 1L
    v  <- match(varname[i], vars)
    if (is.na(p) || p < 1 || p > nperiods) next
    # weighted cumulation: stored temporarily as (weighted_sum, weight)
    # we use two-pass: collect, then divide
  }

  # two-pass: accumulate weighted sums and counts, then divide
  wsum  <- matrix(0.0, nrow = nperiods, ncol = nVar)
  wcnt  <- matrix(0.0, nrow = nperiods, ncol = nVar)
  for (i in seq_along(varname)) {
    p <- period[i] - min_period + 1L
    v <- match(varname[i], vars)
    if (is.na(p) || p < 1 || p > nperiods || is.na(index[i])) next
    wsum[p, v] <- wsum[p, v] + index[i] * n[i]
    wcnt[p, v] <- wcnt[p, v] + n[i]
  }
  has_data <- wcnt > 0
  mat[has_data] <- wsum[has_data] / wcnt[has_data]  # 0 where missing

  # build period descriptor data frame from unique periods
  # we need one row per period index
  period_df <- .build_period_df(period, year, month, min_period, nperiods,
                                agg_interval, min_date, multiple)

  list(mat = mat, var_names = vars, period_df = period_df)
}

#' Build a data frame describing each period
#' @noRd
.build_period_df <- function(period, year, month, min_period, nperiods,
                              agg_interval, min_date, multiple = 1L) {
  min_year  <- as.integer(format(min_date, "%Y"))
  min_month <- as.integer(format(min_date, "%m"))
  min_qtr   <- ceiling(min_month / 3)

  yr  <- integer(nperiods)
  mo  <- integer(nperiods)

  for (i in seq_len(nperiods)) {
    switch(agg_interval,
      annual = {
        yr[i] <- min_year + (i - 1L) * multiple
        mo[i] <- 0L
      },
      multi_year = {
        yr[i] <- min_year + (i - 1L) * multiple
        mo[i] <- 0L
      },
      quarterly = {
        q_abs <- min_qtr + i - 1L
        yr[i] <- min_year + (q_abs - 1L) %/% 4L
        mo[i] <- ((q_abs - 1L) %% 4L + 1L) * 3L  # last month of quarter
      },
      monthly = {
        m_abs <- min_month + i - 1L
        yr[i] <- min_year + (m_abs - 1L) %/% 12L
        mo[i] <- (m_abs - 1L) %% 12L + 1L
      },
      daily = {
        d <- min_date + (i - 1L)
        yr[i] <- as.integer(format(d, "%Y"))
        mo[i] <- as.integer(format(d, "%m"))
      }
    )
  }

  data.frame(period  = seq_len(nperiods),
             year    = yr,
             month   = mo,
             quarter = ifelse(mo > 0L, ceiling(mo / 3L), 0L),
             stringsAsFactors = FALSE)
}

#' Function to perform exponential smoothing - low level function called by extract, not intended to be called by the user. 
#' @noRd
esmooth <-
function(mood, fb, alpha){ 
##########################################################################################
smooth<- function(alpha) { #for time series "series" and alpha "alpha[1]" compute sum of squared forecast error
ferror<- numeric(1)
T<- length(series)
xvect<- numeric(T)        
xvect[1] <-  series[1]
for (t in 2:T) { 
  xvect[t] <-  alpha[1] * series[t] + (1 - alpha[1]) * xvect[t - 1]
}
sumsq <-  0
for (t in 3:T) { 
  ferror <-  series[t] - xvect[t - 1]
  sumsq <-  sumsq + ferror ^ 2
} 
return(sumsq) #this is the value of the function for a particular parameter alpha[1]
} # END OF FUNCTION SMOOTH   
##########################################################################################

series<- mood[fb,] #create series to be smoothed
sm.out<- optim(c(.75),smooth,method="L-BFGS-B",lower=0.5,upper=1)  #call smoother
alpha<- sm.out$par                          #assign result to alpha
#NOW SMOOTH USING ALPHA
T<- length(series)
for (t in 2:T) { 
  mood[fb,t] <-  alpha * series[t] + (1 - alpha) * mood[fb,t - 1]
}
return(alpha)
}

#' Create issue-scale correlation.  Low level function called by extract, not intended to be called by the user. 
#' @noRd
iscorr <-
function(issue,mood) { #compute issue-scale correlations
  Nv<- length(issue[1,])
  Np<- length(issue[,1])
  Rvector<- numeric(Nv)
  for (v in 1:Nv) {
    N<- Np - sum(is.na(issue[,v]))
    if (N > 1) Rvector[v]<- cor(issue[,v],mood[3,],use="complete.obs",method="pearson")
    }
  return(Rvector)
  }

#' Jennings Government Trust Data
#'
#' A dataset of survey marginals from the British Social Attitudes (BSA) survey,
#' measuring public trust in government. These marginals are commonly used as
#' input to the Dyad Ratios Algorithm for constructing latent time series.  We 
#' replaced missing sample sizes with a value of 850, which is roughly the minimum
#' sample size observed in the data. 
#'
#' @format A data frame with 4 variables and `r nrow(jennings)` rows:
#' \describe{
#'   \item{variable}{Character string identifying the survey question or series.}
#'   \item{date}{Date the survey was fielded.}
#'   \item{value}{percentage of people indicating distrust in the government.}
#'   \item{n}{Sample size for the survey wave.}
#' }
#'
#' @source Jennings, W. N. Clarke, J. Moss and G. Stoker  (2017). "The Decline in Diffuse Support for National Politics: The Long View on Political Discontent in Britain" In *Public Opinion Quarterly*,
#'  81(3), 748-758. \doi{10.1093/poq/nfx020}
#'
#' @usage NULL
#' @examples
#' data(jennings)
#' head(jennings)
"jennings"

#' Find periods for aggregation and estimation.  Low-level function called by extract, not intended to be called by the user. 
#' @noRd
findper <-
function(unit,curdate,mind,miny,minper,aggratio) { #returns intFindPer
  datcurdate<-curdate
  class(datcurdate)<-"Date"
  mo <- month(datcurdate)
  qu <- 1 + as.integer((mo - 1)/3)
  dy <- day(datcurdate)
  yr <- year(datcurdate)
  arinv<- 1/aggratio
  if (unit == "D") intFindPer <- curdate - mind +1 #curdate - mindate + 1
  if (unit == "A" || unit == "O") intFindPer <- as.integer((yr - miny) / aggratio) + 1
  if (unit == "Q") part <- qu
  if (unit == "M") part <- mo
  if (unit == "Q" || unit == "M") intFindPer <- (yr - miny - 1) * arinv + part + (arinv - (minper - 1))
  return(intFindPer)
  }

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

dyad_ratios_cpp <- function(data_matrix, nperiods, nVar, npass, smoothing, tola_in, beginper, endper, raw_av, raw_std, seed = 1L) {
    .Call(`_DyadRatios_dyad_ratios_cpp`, data_matrix, nperiods, nVar, npass, smoothing, tola_in, beginper, endper, raw_av, raw_std, seed)
}

#' calculates mmood - low level function called by extract, not intended to be called by the user. 
#' @noRd
dominate <-
function(fb,issue,nperiods,nvar,mood,valid,smoothing,alpha,csign) {
nitems<- numeric(nperiods)
if (fb==2) alpha1<-alpha
if (fb==1) {
  unexp<-numeric(1)
  everlap<- integer(1)
  alpha<- 1
  alpha1<- 1
  } 
  
  if (fb == 1) {
    startper <- 1
    mood[fb, startper] <- 100
    firstj <- 2
    lastj <- nperiods
    stepj <- 1
    jprev <- 1
  } else {
    startper <- nperiods
    mood[fb, startper] <- mood[1, nperiods] #reuse forward metric
    firstj <- nperiods - 1
    lastj <- 1
    stepj <- -1
    jprev <- nperiods
  } #    end if
for (j in seq(firstj,lastj,by=stepj)) {  
  mood[fb, j] <- 0
  everlap <- 0 ## of years which have contributed sums to mood
  if (fb == 1) {
    firstj2 <- 1
    lastj2 <- j - 1
  } else  {
    firstj2 <- j + 1
    lastj2 <- nperiods
  } # end if

  for (j2 in firstj2:lastj2) { 
    sum <- 0     #has already been estimated
    consum <- 0  #sum of communalities across issues
    overlap <- 0
    for (v in 1:nvar) { 
      xj <- issue[j, v]                      #xj is base year value
      sngx2 <- issue[j2, v]                  #sngx2 is comparison year value
      if (!is.na(xj) && !is.na(sngx2)) {  
        overlap <- overlap + 1               #numb of issues contributing to sum
        ratio <- xj / sngx2
        if (csign[v] < 0)  ratio <- 1 / ratio
        sum <- sum + valid[v] * ratio * mood[fb, j2] 
        consum <- consum + valid[v]
      } #              end if
    } #next v
    if (overlap > 0) {
      everlap <- everlap + 1
      mood[fb, j] <- mood[fb, j] + sum / consum
    } # end if
  } #next j2
  nitems[j] <- everlap
  if (everlap > 0) mood[fb, j] <- mood[fb, j] / everlap else mood[fb, j] <- mood[fb, jprev] #if undefined, set to lag(mood)
  jprev <- j #last value of j, whether lead or lag
} #next j
  if (smoothing == TRUE) {
    alpha<- esmooth(mood, fb, alpha)     #NOW SMOOTH USING ALPHA
    mood.sm<- mood[fb,] #set up alternate vector mood.sm
    for (t in 2:nperiods) { 
      mood.sm[t]<- alpha*mood[fb,t]+(1-alpha)*mood.sm[t-1]
    } #end for
    mood[fb,]<- mood.sm #now assign back smoothed version
  } else {
    alpha1 <- 1
    alpha <- 1
  } 
  if (smoothing == TRUE && fb == 1) alpha1 <- alpha
dominate.out<- list(alpha1=alpha1,alpha=alpha,latent=mood[fb,]) #output object
return(dominate.out)  
#  return(mood[fb,])
}

---
title: "Documentation for the Dyad Ratios Algorithm"
author: "James Stimson with light editing and updating by Dave Armstrong"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Documentation for the Dyad Ratios Algorithm}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## General

`extract()` is an R function which implements the dyad ratios algorithm described in Stimson (2018) for constructing a latent time series from the raw materials of survey research marginals. It inputs data in archival form, as dated survey results, aggregates them into regular periods selected at run time, and extracts the latent dimension(s) which best account for the raw series.

Although its central mathematics is entirely different - based upon ratios of comparable observed series rather than covariance of "variables" - its behavior is strongly analogous to extraction of a "factor score" in standard (iterative) principal components analysis.

## Usage

The routine is called from R by the command:

```{r, eval=FALSE}
output<- extract(varname, date, index, ncases=NULL, 
  unit="A", mult=1, begindt=NA, enddt=NA, npass=1,
  smoothing=TRUE, endmonth=12)
```

### Arguments
The first three arguments must be specified by the user, others are optional:

  1. `varname` is an $n$-length vector containing names given to input series. The routine assumes that any two observations with the same name are comparable and that any change in name signals noncomparability. Ratios are computed only between comparable observations.

  2. `date` is an $n$-length vector of dates, typically one of the dates of survey field work (e.g., first, last, or median day). This should be recorded in a date format.  For example, if you had the string "2025-01-01", you could use `lubridate::ymd("2025-01-01")` to convert it to a date format. 

  3. `index` is an $n$-length vector of the numeric summary value of the result. It might be a percent or proportion responding in a single category, (e.g., the "approve" response in presidential approval) or some multi-response summary, for example,

  $$\text{Index} = \frac{\text{Percent Agree}}{\text{Percent Agree} + \text{Percent Disagree}}$$
 
  Interpretation of the derived latent dimension is eased by having the index coded such that polarity is the same across items—for example, if the concept being measured is liberalism, then high values always stand for the more liberal response - but as in principal component analysis, the routine deals appropriately with polarity switches.

  4. `ncases` is an $n$-length vector giving the number of cases for each value in `index`, typically sample size, is used during aggregation to produce a weighted average when multiple readings fall in one aggregation period. If this issue doesn’t occur or if the user prefers an unweighted average, then ncases=NULL—or omitting the ncases variable—will ignore case weighting. In the case of a mix of available and missing ncases indicators, 0 or NA values are reset to 1000.
 
  5. `unit` is the aggregation period, "D" (daily), "M" (monthly), "Q" (quarterly), "A" (annual, default), "O" (other, for multi-year aggregation). `mult` is the number of years, used only for unit option "O".

  6. `begindt` is the beginning date for analysis. Default (`NA`) determines beginning date from the earliest date in the data.  Entries for beginning and ending date use the ISOdate function. For example, to start an analysis in January, 1993, enter `ISOdate(1993,1,1)`. (As always, R is case sensitive. So "ISO" must be caps and "date" lower case.)
 
  7. `enddt` is the ending date for analysis. Default (`NA`) determines ending date from the latest date in the data.
  
  Warning: The routine cannot determine the earliest or latest dates of items which not actually are used in the analysis. The criterion for usage is that items must appear in more than one period *after* aggregation. So if the beginning or ending dates are determined by an item which is discarded because it does not meet this criterion, the routine will fail.

  8. `smoothing` specifies whether or not exponential smoothing is applied to intermediate estimates during the iterative solution process. Default is TRUE. 
  
  9. `npass` number of dimensions to be extracted can be 1 or 2, defaults to 1. 



##	Dating Considerations

The routine expects a date variable of R class `date`. Generally, import packages like `foreign`, `readr`, `haven`, `rio` will try to turn date-looking variables into date objects.  The `str()` function will identify if your variable is a date or character.  We can see from the output below that the `date` variable in the `jennings` dataset is indeed in a `date` format. 

```{r}
library(DyadRatios)
data(jennings)
str(jennings$date)
```

If your variable is not already in a date format, you can transform it as such with the functions from the `lubridate` package.  For example, if your date variable is in the format "2025-01-01", you can use the `ymd()` function to convert it to a date format as follows:

```{r}
library(lubridate)
temp <- data.frame(survey_date = c("2025-01-01", "2024-10-13", "2020-05-13"))
str(temp)
temp$survey_date <- ymd(temp$survey_date)
str(temp)
```
See how the class changed from `chr` to `Date`.  

If you happened to have three variables for month, day and year, you could paste them together and use the `lubridate` functions: 

```{r}
temp <- data.frame(year= c(2025, 2024, 2020), 
                   month = c(1, 10, 5), 
                   day = c(1, 13, 13))
temp$date <- lubridate::ymd(paste(temp$year, temp$month, temp$day, sep="-"))
str(temp)
```


Warning: The `lubridate` functions will not handle fake dates (for example, 1-32-05). It decodes dates that actually existed on past calendars or will exist on future ones (e.g., no Feb 29 unless year is actually a leap year.)

Ideally, the dates will be in a structured format.  The lubridate functions will even parse string dates with words, e.g., "January 1, 1993" or "Aug 2, 2020" so long as the strings have month, day and year in the same position in the date.  

```{r}
temp <- data.frame(date = c("January 1, 1993", "Jan 1, 1993", "Aug 2, 2020"))
temp$date <- mdy(temp$date)
```

The formats produced by importing excel or csv documents will not be identical to those produced by `lubridate` functions, but they will still work with the `extract()` function as their components can still be extracted by the `lubridate()` functions `month()`, `day()` and `year()`. 


```{r}
temp <- data.frame(date = ymd("2025-01-01", "1990-01-01", "1970-01-01", "1950-01-01"))
xl_temp <- tempfile(fileext = ".xlsx")
csv_temp <- tempfile(fileext = ".csv")

rio::export(temp, xl_temp)
rio::export(temp, csv_temp)


temp_xl <- rio::import(xl_temp)
temp_csv <- rio::import(csv_temp)

str(temp_xl)
str(temp_csv)

```
Notice that from excel, you get a `POSIXct` format variable and from csv you get an `IDate` object.  They still are equal in integer form to the `lubridate` way.  All values pass the equivalence test. 

```{r}
all(temp$date == temp_xl$date)
all(temp$date == temp_csv$date)
```

## Output

The `extract` function produces as output 8 categories of information:
 
  1. `formula` reproduces the user call
  2. `setup` supplies basic information about options and the iterative solution
  3. `period` is a list of the aggregation periods, for example, 2005.02 for February, 2005
  4. `varname` is a list in order of the variables actually used in the analysis, a subset of all those in the data.
  5. `loadings` are the item-scale correlations from the final solution. Their square is the validity estimate used in weighting.
  6. `means` and `std.deviations` document the item descriptive information, and
  7. `latent` is the estimated time series, the purpose of everything. The variable `latent1` is for the first dimension and `latent2` for the second, if applicable. 
  8. Diagnostic Information: there are several additional objects returned in the output, including `hist` - the iteration history, `totalvar` - the total variance to be explained, 
  `var_exp` - the variance explained by each dimension, `dropped` - any series dropped from the analysis, and `smoothing` - whether smoothing was applied during the iterative solution process.

### Output Functions

The raw output object created at run time contains everything of interest, but in an inconvenient format. There are several functions that can be used to display results in different ways. 

  1. `plot` displays a time series ggplot of the number of dimensions estimated on y axes against time units on the x axis and can be called with `plot(object)`
  
  2. `print` displays the output of the iterative solution process.  This is what previously was printed with `verbose=TRUE` which has been removed in favor of a print method for the function output. 
  
  3. `summary` displays information about the raw series of the analysis. Under the heading: "Variable Name Cases Loading Mean Std Dev" it lists as many series as are used in the solution, giving variable name, number of cases (after aggregation), dimension loadings, and means and standard deviations of the raw series.  
  
  4. `get_mood` retrieves the period and latent dimension estimate(s) from the model object and returns them in a data frame. 
 
###	Negative Correlations?

Correlations, in the case of time series, measure whether two series vary in or out of phase. Thus the cross-sectional interpretation of the negative correlation—that two items are inversely related - does not hold. It is not unusual to observe negative "loadings" in extract analyses. They mean only that items move out of phase, not that they are opposites.


## Model

Assume $N$ survey results, each coded into a meaningful single indicator. These results consist of $n$ subsets of comparable items measured at different times, $t={1,\ldots, T}$. Designate each result $x_{it}$, where the $i$ subscript indicates item and $t$ indicates aggregation period, $1,\ldots, T$.

The starting assumption is that ratios, $r_{it+k} = \frac{x_it+k}{x_{it}}$ of the comparable item $i$ at different times will be meaningful indicators of the latent concept to be extracted. Metric information is thus lost, which is desirable because absent a science of question wording, we have no ability to compare different items. If there were no missing observations, then for each item $i$, we could let $r_{i1}=1.0$ and observe the complete set of ratios, $r_{i2}, r_{i3}, \ldots, r_{iT}$. Then an average across the n items forms an excellent estimate of the latent concept $\theta_{t}$:
 
$$\theta_{t} = \frac{\sum_{i=1}^{n}r_{it}}{n}$$

But we do have missing x's - and in the typical case it is most of them. We would still be in good shape if we had a common period, say period 1, which was available for all series. We could then form the sum from the $k$ available items, $k\leq n$, and divide by $k$. But we also lack such a common period. That motivates a recursive approach.

**Forward Recursion**:	Begin by selecting that subset of items which are available at time 1. For them we can form $\hat{\theta}$ for $t=1,\ldots, T$ setting $\hat{\theta}_{1} = 1.0$ and calculating $\hat{\theta}_{2}, \ldots, \hat{\theta}_{T}$ from whatever subsets of items are available.  Now proceed to period 2, setting $\hat{\theta}_{2}$ to that value estimated from period 1 and now, using the subset of items which include period 2, estimating $\hat{\theta}_{3}, \ldots, \hat{\theta}_{T}$ from the assumption that  $\theta_{2} = \hat{\theta}_{2}$.  By projecting $\hat{\theta}_{2}$ forward in this manner,  the estimates for periods 3 through $T$ become comparable to what they would have been had period 1 information been available. This procedure is repeated one period at a time through $T-1$, giving from 1 to $T-1$ different estimates of each of the $\theta_{t}$. An average of all of them becomes $\hat{\theta}_{t}$.

**Backward Recursion**: It will be seen that forward recursion very heavily weights early information relative to later information. Period 1 contributes to all subsequent estimates,whereas period $T-1$ contributes only to $T$, and period $T$ only to itself. Thus the direction of recursion matters. Employing the same procedure backward puts a different weight on the items and gives a comparable, but not identical, set of estimates. Thus a more efficient set of estimates, one weighting all information equally, can be gained by averaging the two recursive estimates. (And the correlation between forward and backward series becomes a reliability estimate.)

### Iterative Validity Estimation

As in iterated principal components we both make assumptions about item validity and then, post hoc, have the ability to observe empirical estimates of validities (the square of item/scale correlations). At the beginning validities are assumed to be 1.0 for all items. Then the empirically estimated validities become assumed validities for the next iteration. This procedure is repeated until the difference between assumed and estimated validities is effectively zero for all items, the maximum item discrepancy less than .001.

## Example

We illustrate the use of `extract` with the data from Jennings et. al. (2017) who graciously shared their data through dataverse.  The first few observations show the question names, dates, percentage distrusting in government, and sample size. 

```{r}
library(DyadRatios)
data(jennings)
head(jennings)
```

We could see how many questions there are in the data. 

```{r}
library(dplyr) 
jennings %>% 
  group_by(variable) %>% 
  tally() %>% 
  arrange(desc(n)) %>% 
  print(n=40)
```

We can now estimate the dyad ratios algorithm: 

```{r}
jennings_out <- DyadRatios::extract(
  varname = jennings$variable, 
  date = jennings$date, 
  index = jennings$value, 
  ncases = jennings$n, 
  begindt = lubridate::ymd("1985-01-01", tz="GMT"), 
  enddt = max(jennings$date), 
  npass=1
)
```

We can print the information about the estimates with: 

```{r}
print(jennings_out)
```

Our latent dimension explains about $50\%$ of the variance in the raw series.  We could also look at the loadings: 

```{r}
summary(jennings_out)
```

Some of these have much higher loadings than others.  That means, they are more reliable indicators of government (dis)trust than others.  For example, `govtrust_bsa` (actual question from the BSA: "Do not trust British governments to place needs of the nation above interests of their own party?") is a very reliable indicator with observations in 20 different time-periods.  On the other hand, `bse_MPs` (actual question from the BSA: "Those we elect as MPs lose touch with people pretty quickly") has a quite lower loading indicating it is a much less reliable indicator of (dis)trust.  We can also make a plot of mood: 

```{r out.width="75%", fig.align="center", fig.height=5, fig.width=8}
plot(jennings_out)
```

Finally, to retrieve the latent dimension and the periods, we can use the `get_mood` function:

```{r}
ests <- get_mood(jennings_out)
ests
```


## Bootstrapping 

Dave Armstrong added this facet of the analytical framework.  

There is no inferential framework baked-in to Stimson's original method.  In fact, in my conversations with Jim about this point, he seemed wholly uninterested in the sampling uncertainty of the measure.  Given that I think Jim has forgotten more about this stuff than I know, he may well have the right of it.  That said, I think it is interesting, so I decided to write a bootstrapping algorightm to get a sense of the variability of the estimates.  I would argue that we should not do a simple non-parametric bootstrap because that will change the temporal density of the data.  Instead, we could use a parametric bootstrap.  This allows us to get a sense of how variable the estimates are given the data that we have.  The algorithm uses the following steps. 

1. Sample the number of cases as $n^{(b)}_i \sim \text{Binomial}(\text{index}_i, \text{ncases}_i)$ for each observation in the model.  
2. Calculate the boostrapped index value $\text{index}_i^{(b)} = \frac{n^{(b)}_i}{\text{ncases}_i}$. 
3. Estimate mood using $\text{index}_{i}^{(b)}$ and the original number of cases.  In this step, when the data are normalized (i.e., the weighted mean and standard deviation are calculated), we use the weighted mean and standard deviation from the original data so that differences in the mean and standard deviation are not responsible for the variation.
4. Repeat steps 1-3 for $R$ bootstrap samples.
5. Calculate the $\frac{1-\text{level}}{2}$ and $1-\frac{1-\text{level}}{2}$ quantiles of the bootstrap distribution.

Here's the bootstrapped analysis for the Jennings data. 

```{r}
data(jennings)
jennings_boot <- boot_dr(
  varname = jennings$variable, 
  date = jennings$date, 
  index = jennings$value, 
  ncases = jennings$n, 
  begindt = lubridate::ymd("1985-01-01"), 
  enddt = max(jennings$date), 
  npass=1, 
  R=1000, 
  parallel=FALSE, 
  pw = TRUE
)
```

The bootstrapped results (if `pw=TRUE`) contains two elements - `ci` and `pw`.  The `ci` element contains the estimate and confidence intervals for the latent dimension.  Here are the first few lines of the confidence intervals: 

```{r}
head(jennings_boot$ci)
```

The main use for this would be a plot like the following: 

```{r out.width="75%", fig.align="center", fig.height=5, fig.width=8}
library(ggplot2)
ggplot(jennings_boot$ci, aes(x=period)) + 
  geom_ribbon(aes(ymin=lwr, ymax=upr), alpha=0.2) + 
  geom_line(aes(y=latent1)) + 
  labs(y="Mood (Distrust in Government)", x="Year") + 
  theme_bw()
```

If `pw=TRUE`, the `pw` element contains pairwise comparisons between all time periods.  This would be most useful for identifying where "significant" changes in mood happen.  For example, you could look at the significant differences, we'll just look at the first few here: 

```{r}
jennings_boot$pw %>% 
  filter(p_diff > .95) %>% 
  head()
```

You could also make a plot of which differences are significant for all pairs. 

```{r out.width="75%", fig.align="center", fig.height=8, fig.width=8}
library(tidyr)
pwdiff <- jennings_boot$pw %>% 
  mutate(diff_sig = ifelse(p_diff > .95, diff, 0))

ggplot(pwdiff, aes(x=as.factor(p1), y=as.factor(p2), fill=diff_sig)) + 
  geom_tile(color="black") + 
  scale_fill_gradient2(low="#377eb8", mid="white", high="#e41a1c", na.value = "grey90") +
  labs(x="From", y="To", fill = "To - From") + 
  theme_classic() + 
  theme(axis.text.x = element_text(angle=90, vjust=0.5, hjust=1), 
        legend.position = "inside", 
        legend.position.inside = c(.85, .33))


```
In this instance, anywhere that is white indicates a non-significant difference.  We can see from the plot above that there are many year-over-year changes that are statistically significant.  We could print them out, arranged by size: 

```{r}
jennings_boot$pw %>% 
  filter(p_diff > .95 & p2 - p1 == 1) %>% 
  arrange(diff)
```




## References

Jennings, W. N. Clarke, J. Moss and G. Stoker  (2017). "The Decline in Diffuse Support for National Politics: The Long View on Political Discontent in Britain", Public Opinion Quarterly, 81(3), 748-758. \doi{10.1093/poq/nfx020}

Stimson, James A. 2018. "The Dyad Ratios Algorithm for Estimating Latent Public Opinion: Estimation, Testing, and Comparison to Other Approaches." Bulletin of Methodological Sociology 137-138:201–218. \doi{doi:10.1177/0759106318761614}

---
title: "Documentation for the Dyad Ratios Algorithm"
author: "James Stimson with light editing and updating by Dave Armstrong"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Documentation for the Dyad Ratios Algorithm}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## General

`extract()` is an R function which implements the dyad ratios algorithm described in Stimson (2018) for constructing a latent time series from the raw materials of survey research marginals. It inputs data in archival form, as dated survey results, aggregates them into regular periods selected at run time, and extracts the latent dimension(s) which best account for the raw series.

Although its central mathematics is entirely different - based upon ratios of comparable observed series rather than covariance of "variables" - its behavior is strongly analogous to extraction of a "factor score" in standard (iterative) principal components analysis.

## Usage

The routine is called from R by the command:

```{r, eval=FALSE}
output<- extract(varname, date, index, ncases=NULL, 
  unit="A", mult=1, begindt=NA, enddt=NA, npass=1,
  smoothing=TRUE, endmonth=12)
```

### Arguments
The first three arguments must be specified by the user, others are optional:

  1. `varname` is an $n$-length vector containing names given to input series. The routine assumes that any two observations with the same name are comparable and that any change in name signals noncomparability. Ratios are computed only between comparable observations.

  2. `date` is an $n$-length vector of dates, typically one of the dates of survey field work (e.g., first, last, or median day). This should be recorded in a date format.  For example, if you had the string "2025-01-01", you could use `lubridate::ymd("2025-01-01")` to convert it to a date format. 

  3. `index` is an $n$-length vector of the numeric summary value of the result. It might be a percent or proportion responding in a single category, (e.g., the "approve" response in presidential approval) or some multi-response summary, for example,

  $$\text{Index} = \frac{\text{Percent Agree}}{\text{Percent Agree} + \text{Percent Disagree}}$$
 
  Interpretation of the derived latent dimension is eased by having the index coded such that polarity is the same across items—for example, if the concept being measured is liberalism, then high values always stand for the more liberal response - but as in principal component analysis, the routine deals appropriately with polarity switches.

  4. `ncases` is an $n$-length vector giving the number of cases for each value in `index`, typically sample size, is used during aggregation to produce a weighted average when multiple readings fall in one aggregation period. If this issue doesn’t occur or if the user prefers an unweighted average, then ncases=NULL—or omitting the ncases variable—will ignore case weighting. In the case of a mix of available and missing ncases indicators, 0 or NA values are reset to 1000.
 
  5. `unit` is the aggregation period, "D" (daily), "M" (monthly), "Q" (quarterly), "A" (annual, default), "O" (other, for multi-year aggregation). `mult` is the number of years, used only for unit option "O".

  6. `begindt` is the beginning date for analysis. Default (`NA`) determines beginning date from the earliest date in the data.  Entries for beginning and ending date use the ISOdate function. For example, to start an analysis in January, 1993, enter `ISOdate(1993,1,1)`. (As always, R is case sensitive. So "ISO" must be caps and "date" lower case.)
 
  7. `enddt` is the ending date for analysis. Default (`NA`) determines ending date from the latest date in the data.
  
  Warning: The routine cannot determine the earliest or latest dates of items which not actually are used in the analysis. The criterion for usage is that items must appear in more than one period *after* aggregation. So if the beginning or ending dates are determined by an item which is discarded because it does not meet this criterion, the routine will fail.

  8. `smoothing` specifies whether or not exponential smoothing is applied to intermediate estimates during the iterative solution process. Default is TRUE. 
  
  9. `npass` number of dimensions to be extracted can be 1 or 2, defaults to 1. 



##	Dating Considerations

The routine expects a date variable of R class `date`. Generally, import packages like `foreign`, `readr`, `haven`, `rio` will try to turn date-looking variables into date objects.  The `str()` function will identify if your variable is a date or character.  We can see from the output below that the `date` variable in the `jennings` dataset is indeed in a `date` format. 

```{r}
library(DyadRatios)
data(jennings)
str(jennings$date)
```

If your variable is not already in a date format, you can transform it as such with the functions from the `lubridate` package.  For example, if your date variable is in the format "2025-01-01", you can use the `ymd()` function to convert it to a date format as follows:

```{r}
library(lubridate)
temp <- data.frame(survey_date = c("2025-01-01", "2024-10-13", "2020-05-13"))
str(temp)
temp$survey_date <- ymd(temp$survey_date)
str(temp)
```
See how the class changed from `chr` to `Date`.  

If you happened to have three variables for month, day and year, you could paste them together and use the `lubridate` functions: 

```{r}
temp <- data.frame(year= c(2025, 2024, 2020), 
                   month = c(1, 10, 5), 
                   day = c(1, 13, 13))
temp$date <- lubridate::ymd(paste(temp$year, temp$month, temp$day, sep="-"))
str(temp)
```


Warning: The `lubridate` functions will not handle fake dates (for example, 1-32-05). It decodes dates that actually existed on past calendars or will exist on future ones (e.g., no Feb 29 unless year is actually a leap year.)

Ideally, the dates will be in a structured format.  The lubridate functions will even parse string dates with words, e.g., "January 1, 1993" or "Aug 2, 2020" so long as the strings have month, day and year in the same position in the date.  

```{r}
temp <- data.frame(date = c("January 1, 1993", "Jan 1, 1993", "Aug 2, 2020"))
temp$date <- mdy(temp$date)
```

The formats produced by importing excel or csv documents will not be identical to those produced by `lubridate` functions, but they will still work with the `extract()` function as their components can still be extracted by the `lubridate()` functions `month()`, `day()` and `year()`. 


```{r}
temp <- data.frame(date = ymd("2025-01-01", "1990-01-01", "1970-01-01", "1950-01-01"))
xl_temp <- tempfile(fileext = ".xlsx")
csv_temp <- tempfile(fileext = ".csv")

rio::export(temp, xl_temp)
rio::export(temp, csv_temp)


temp_xl <- rio::import(xl_temp)
temp_csv <- rio::import(csv_temp)

str(temp_xl)
str(temp_csv)

```
Notice that from excel, you get a `POSIXct` format variable and from csv you get an `IDate` object.  They still are equal in integer form to the `lubridate` way.  All values pass the equivalence test. 

```{r}
all(temp$date == temp_xl$date)
all(temp$date == temp_csv$date)
```

## Output

The `extract` function produces as output 8 categories of information:
 
  1. `formula` reproduces the user call
  2. `setup` supplies basic information about options and the iterative solution
  3. `period` is a list of the aggregation periods, for example, 2005.02 for February, 2005
  4. `varname` is a list in order of the variables actually used in the analysis, a subset of all those in the data.
  5. `loadings` are the item-scale correlations from the final solution. Their square is the validity estimate used in weighting.
  6. `means` and `std.deviations` document the item descriptive information, and
  7. `latent` is the estimated time series, the purpose of everything. The variable `latent1` is for the first dimension and `latent2` for the second, if applicable. 
  8. Diagnostic Information: there are several additional objects returned in the output, including `hist` - the iteration history, `totalvar` - the total variance to be explained, 
  `var_exp` - the variance explained by each dimension, `dropped` - any series dropped from the analysis, and `smoothing` - whether smoothing was applied during the iterative solution process.

### Output Functions

The raw output object created at run time contains everything of interest, but in an inconvenient format. There are several functions that can be used to display results in different ways. 

  1. `plot` displays a time series ggplot of the number of dimensions estimated on y axes against time units on the x axis and can be called with `plot(object)`
  
  2. `print` displays the output of the iterative solution process.  This is what previously was printed with `verbose=TRUE` which has been removed in favor of a print method for the function output. 
  
  3. `summary` displays information about the raw series of the analysis. Under the heading: "Variable Name Cases Loading Mean Std Dev" it lists as many series as are used in the solution, giving variable name, number of cases (after aggregation), dimension loadings, and means and standard deviations of the raw series.  
  
  4. `get_mood` retrieves the period and latent dimension estimate(s) from the model object and returns them in a data frame. 
 
###	Negative Correlations?

Correlations, in the case of time series, measure whether two series vary in or out of phase. Thus the cross-sectional interpretation of the negative correlation—that two items are inversely related - does not hold. It is not unusual to observe negative "loadings" in extract analyses. They mean only that items move out of phase, not that they are opposites.


## Model

Assume $N$ survey results, each coded into a meaningful single indicator. These results consist of $n$ subsets of comparable items measured at different times, $t={1,\ldots, T}$. Designate each result $x_{it}$, where the $i$ subscript indicates item and $t$ indicates aggregation period, $1,\ldots, T$.

The starting assumption is that ratios, $r_{it+k} = \frac{x_it+k}{x_{it}}$ of the comparable item $i$ at different times will be meaningful indicators of the latent concept to be extracted. Metric information is thus lost, which is desirable because absent a science of question wording, we have no ability to compare different items. If there were no missing observations, then for each item $i$, we could let $r_{i1}=1.0$ and observe the complete set of ratios, $r_{i2}, r_{i3}, \ldots, r_{iT}$. Then an average across the n items forms an excellent estimate of the latent concept $\theta_{t}$:
 
$$\theta_{t} = \frac{\sum_{i=1}^{n}r_{it}}{n}$$

But we do have missing x's - and in the typical case it is most of them. We would still be in good shape if we had a common period, say period 1, which was available for all series. We could then form the sum from the $k$ available items, $k\leq n$, and divide by $k$. But we also lack such a common period. That motivates a recursive approach.

**Forward Recursion**:	Begin by selecting that subset of items which are available at time 1. For them we can form $\hat{\theta}$ for $t=1,\ldots, T$ setting $\hat{\theta}_{1} = 1.0$ and calculating $\hat{\theta}_{2}, \ldots, \hat{\theta}_{T}$ from whatever subsets of items are available.  Now proceed to period 2, setting $\hat{\theta}_{2}$ to that value estimated from period 1 and now, using the subset of items which include period 2, estimating $\hat{\theta}_{3}, \ldots, \hat{\theta}_{T}$ from the assumption that  $\theta_{2} = \hat{\theta}_{2}$.  By projecting $\hat{\theta}_{2}$ forward in this manner,  the estimates for periods 3 through $T$ become comparable to what they would have been had period 1 information been available. This procedure is repeated one period at a time through $T-1$, giving from 1 to $T-1$ different estimates of each of the $\theta_{t}$. An average of all of them becomes $\hat{\theta}_{t}$.

**Backward Recursion**: It will be seen that forward recursion very heavily weights early information relative to later information. Period 1 contributes to all subsequent estimates,whereas period $T-1$ contributes only to $T$, and period $T$ only to itself. Thus the direction of recursion matters. Employing the same procedure backward puts a different weight on the items and gives a comparable, but not identical, set of estimates. Thus a more efficient set of estimates, one weighting all information equally, can be gained by averaging the two recursive estimates. (And the correlation between forward and backward series becomes a reliability estimate.)

### Iterative Validity Estimation

As in iterated principal components we both make assumptions about item validity and then, post hoc, have the ability to observe empirical estimates of validities (the square of item/scale correlations). At the beginning validities are assumed to be 1.0 for all items. Then the empirically estimated validities become assumed validities for the next iteration. This procedure is repeated until the difference between assumed and estimated validities is effectively zero for all items, the maximum item discrepancy less than .001.

## Example

We illustrate the use of `extract` with the data from Jennings et. al. (2017) who graciously shared their data through dataverse.  The first few observations show the question names, dates, percentage distrusting in government, and sample size. 

```{r}
library(DyadRatios)
data(jennings)
head(jennings)
```

We could see how many questions there are in the data. 

```{r}
library(dplyr) 
jennings %>% 
  group_by(variable) %>% 
  tally() %>% 
  arrange(desc(n)) %>% 
  print(n=40)
```

We can now estimate the dyad ratios algorithm: 

```{r}

jennings_out <- DyadRatios::extract(
  jennings,  
  varname_col = "variable", 
  date_col = "date", 
  index_col = "value", 
  n_col = "n", 
  agg_interval = "annual", 
  start_date = lubridate::ymd("1985-01-01", tz="GMT"), 
  end_date = max(jennings$date), 
  n_dim = 1, 
  smoothing = TRUE)
```

We can print the information about the estimates with: 

```{r}
print(jennings_out)
```

Our latent dimension explains about $50\%$ of the variance in the raw series.  We could also look at the loadings: 

```{r}
summary(jennings_out)
```

Some of these have much higher loadings than others.  That means, they are more reliable indicators of government (dis)trust than others.  For example, `govtrust_bsa` (actual question from the BSA: "Do not trust British governments to place needs of the nation above interests of their own party?") is a very reliable indicator with observations in 20 different time-periods.  On the other hand, `bse_MPs` (actual question from the BSA: "Those we elect as MPs lose touch with people pretty quickly") has a quite lower loading indicating it is a much less reliable indicator of (dis)trust.  We can also make a plot of mood: 

```{r out.width="75%", fig.align="center", fig.height=5, fig.width=8}
plot(jennings_out)
```

Finally, to retrieve the latent dimension and the periods, we can use the `get_mood` function:

```{r}
ests <- get_mood(jennings_out)
ests
```


## Bootstrapping 

Dave Armstrong added this facet of the analytical framework.  

There is no inferential framework baked-in to Stimson's original method.  In fact, in my conversations with Jim about this point, he seemed wholly uninterested in the sampling uncertainty of the measure.  Given that I think Jim has forgotten more about this stuff than I know, he may well have the right of it.  That said, I think it is interesting, so I decided to write a bootstrapping algorightm to get a sense of the variability of the estimates.  I would argue that we should not do a simple non-parametric bootstrap because that will change the temporal density of the data.  Instead, we could use a parametric bootstrap.  This allows us to get a sense of how variable the estimates are given the data that we have.  The algorithm uses the following steps. 

1. Sample the number of cases as $n^{(b)}_i \sim \text{Binomial}(\text{index}_i, \text{ncases}_i)$ for each observation in the model.  
2. Calculate the boostrapped index value $\text{index}_i^{(b)} = \frac{n^{(b)}_i}{\text{ncases}_i}$. 
3. Estimate mood using $\text{index}_{i}^{(b)}$ and the original number of cases.  In this step, when the data are normalized (i.e., the weighted mean and standard deviation are calculated), we use the weighted mean and standard deviation from the original data so that differences in the mean and standard deviation are not responsible for the variation.
4. Repeat steps 1-3 for $R$ bootstrap samples.
5. Calculate the $\frac{1-\text{level}}{2}$ and $1-\frac{1-\text{level}}{2}$ quantiles of the bootstrap distribution.

Here's the bootstrapped analysis for the Jennings data. 

```{r}
data(jennings)
jennings_boot <- boot_dr(
  obj = jennings_out, 
  data = jennings, 
  R=50L, pw=TRUE)
```

The bootstrapped results (if `pw=TRUE`) contains two elements - `ci` and `pw`.  The `ci` element contains the estimate and confidence intervals for the latent dimension.  Here are the first few lines of the confidence intervals: 

```{r}
head(jennings_boot$estimates)
```

The main use for this would be a plot like the following: 

```{r out.width="75%", fig.align="center", fig.height=5, fig.width=8}
library(ggplot2)
ggplot(jennings_boot$estimates, aes(x=year, y=mood, ymin = lower, ymax=upper)) + 
  geom_ribbon( alpha=0.2) + 
  geom_line() + 
  labs(y="Mood (Distrust in Government)", x="Year") + 
  theme_bw()
```

If `pw=TRUE`, the `pw` element contains pairwise comparisons between all time periods.  This would be most useful for identifying where "significant" changes in mood happen.  For example, you could look at the significant differences, we'll just look at the first few here: 

```{r}
jennings_boot$pw %>% 
  filter(p_diff > .95) %>% 
  head()
```

You could also make a plot of which differences are significant for all pairs. 

```{r out.width="75%", fig.align="center", fig.height=8, fig.width=8}
library(tidyr)
pwdiff <- jennings_boot$pw %>% 
  mutate(diff_sig = ifelse(p_diff > .95, diff, 0))

ggplot(pwdiff, aes(x=as.factor(p1), y=as.factor(p2), fill=diff_sig)) + 
  geom_tile(color="black") + 
  scale_fill_gradient2(low="#377eb8", mid="white", high="#e41a1c", na.value = "grey90") +
  labs(x="From", y="To", fill = "To - From") + 
  theme_classic() + 
  theme(axis.text.x = element_text(angle=90, vjust=0.5, hjust=1), 
        legend.position = "inside", 
        legend.position.inside = c(.85, .33))


```
In this instance, anywhere that is white indicates a non-significant difference.  We can see from the plot above that there are many year-over-year changes that are statistically significant.  We could print them out, arranged by size: 

```{r}
jennings_boot$pw %>% 
  filter(p_diff > .95 & p2 - p1 == 1) %>% 
  arrange(diff)
```




## References

Jennings, W. N. Clarke, J. Moss and G. Stoker  (2017). "The Decline in Diffuse Support for National Politics: The Long View on Political Discontent in Britain", Public Opinion Quarterly, 81(3), 748-758. \doi{10.1093/poq/nfx020}

Stimson, James A. 2018. "The Dyad Ratios Algorithm for Estimating Latent Public Opinion: Estimation, Testing, and Comparison to Other Approaches." Bulletin of Methodological Sociology 137-138:201–218. \doi{doi:10.1177/0759106318761614}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dyad_ratios.R
\name{extract}
\alias{extract}
\title{Run the Dyad Ratios Algorithm}
\usage{
extract(
  data,
  varname_col = "varname",
  date_col = "date",
  index_col = "index",
  n_col = NULL,
  agg_interval = c("annual", "quarterly", "monthly", "daily", "multi_year"),
  multiple = 1L,
  start_date = NULL,
  end_date = NULL,
  n_dim = 1L,
  smoothing = TRUE,
  tol = 0.001,
  fiscal_year_end = 12L,
  smooth_seed = 1L
)
}
\arguments{
\item{data}{A \code{data.frame} with at minimum the columns specified in
\code{varname_col}, \code{date_col}, \code{index_col}, and optionally
\code{n_col}.  Each row represents one survey marginal: the proportion
(or count) of respondents taking a particular position on one issue at
one point in time.  The variable name column identifies which opinion
item the row belongs to.}

\item{varname_col}{Character.  Name of the column that identifies the
opinion variable (issue series).  Default \code{"varname"}.}

\item{date_col}{Character.  Name of the column containing the observation
date.  Must be coercible to \code{Date} via \code{as.Date()}.
Default \code{"date"}.}

\item{index_col}{Character.  Name of the column containing the survey
marginal value (e.g. proportion liberal, per cent approving, etc.).
Default \code{"index"}.}

\item{n_col}{Character or \code{NULL}.  Name of the column containing the
number of respondents (sample size / weight).  When \code{NULL} (default)
all observations receive equal weight of 1000, matching the original
program's default.}

\item{agg_interval}{Character.  Temporal aggregation level.  One of
\code{"annual"} (default), \code{"quarterly"}, \code{"monthly"},
\code{"daily"}, or \code{"multi_year"}.}

\item{multiple}{Integer.  Number of years per period when
\code{agg_interval = "multi_year"}.  Default \code{1}.}

\item{start_date}{Optional \code{Date} (or character coercible by
\code{as.Date}) giving the earliest date to include.  Default: the
earliest date found in \code{data}.}

\item{end_date}{Optional \code{Date} (or character coercible by
\code{as.Date}) giving the latest date to include.  Default: the
latest date found in \code{data}.}

\item{n_dim}{Integer 1 or 2.  Number of latent dimensions to extract.
Default \code{1}.}

\item{smoothing}{Logical.  Apply optimal exponential smoothing to each
forward and backward pass.  Default \code{TRUE}.}

\item{tol}{Numeric.  Convergence tolerance (maximum weighted change in
item-mood correlations between iterations).  Default \code{0.001}.}

\item{fiscal_year_end}{Integer 1–12.  Final month of the fiscal / policy
year; use 12 (default) for calendar years.  When set to a value < 12
observations after this month are rolled forward into the next year,
consistent with the original program's \code{FinalMonth} parameter.}

\item{smooth_seed}{Integer.  Seed for the C random number generator used
inside the exponential-smoothing optimiser (\code{eSmooth}).  The default
value of \code{1} replicates MCalc's implicit behaviour: MCalc never
calls \code{srand()}, so C's \code{rand()} starts from seed 1 on every
run.  Change this only if you need to explore sensitivity to the smoothing
optimiser's random fallback step.  Has no effect when
\code{smoothing = FALSE}.}
}
\value{
An object of class \code{"extract"}, which is a named list
  containing:
  \describe{
    \item{\code{mood}}{Numeric vector of estimated public mood, one value
      per period.  When \code{n_dim = 2} this is the first dimension.}
    \item{\code{mood_dim2}}{Numeric vector of the second dimension, or
      \code{NA} when \code{n_dim = 1}.}
    \item{\code{periods}}{A \code{data.frame} describing each period
      (year, month/quarter where applicable) matching the length of
      \code{mood}.}
    \item{\code{loadings}}{A \code{data.frame} of variable names with
      loading on each extracted dimension.}
    \item{\code{iterations}}{A \code{data.frame} of the iteration history
      (convergence, reliability, smoothing alphas per iteration).}
    \item{\code{n_series}}{Integer.  Number of opinion series retained
      after dropping constant series.}
    \item{\code{n_periods}}{Integer.  Number of time periods.}
    \item{\code{n_obs}}{Integer.  Total number of input observations used.}
    \item{\code{alpha_F}}{Final forward-pass smoothing parameter.}
    \item{\code{alpha_B}}{Final backward-pass smoothing parameter.}
    \item{\code{eigenvalue}}{Eigenvalue estimate for the first dimension.}
    \item{\code{variance_explained}}{Proportion of total variance accounted
      for by the first dimension.}
    \item{\code{variance_explained_dim2}}{(\code{n_dim = 2} only) Additional
      proportion of total variance accounted for by the second dimension (i.e.
      its share of the variance left unexplained by dimension 1).  \code{NA}
      for single-dimension solutions.}
    \item{\code{call}}{The matched \code{\link[base]{call}}.}
    \item{\code{settings}}{List of all parameter values used.}
  }
}
\description{
Estimates one or two latent opinion dimensions from a collection of
time-series issue variables using the Dyad Ratios algorithm developed by
James Stimson (Stimson 1991, 1999).  This function accepts already-loaded
data as a \code{data.frame}, handles aggregation to the requested interval,
standardises the issue matrix, passes it to the compiled C++ core, and
returns a richly-annotated result object.
}
\details{
The algorithm iterates between a forward pass and a backward pass.  In each
pass every period's latent score is estimated as the weighted average of
ratios of issue values relative to all other periods for which both values
are non-missing.  The weights are the squared correlations of each issue
with the current mood estimate.  Convergence is declared when the maximum
weighted change in these correlations falls below \code{tol}.

Issues are standardised to a mean of 100 and a standard deviation of 10
before estimation, then the final mood series is rescaled to have the
weighted mean and standard deviation of the raw issue series.
}
\examples{
# Minimal synthetic example
set.seed(42)
n <- 60
dates <- seq(as.Date("1980-01-01"), by = "year", length.out = n / 3)
dat <- data.frame(
  varname = rep(c("item_a", "item_b", "item_c"), each = length(dates)),
  date    = rep(dates, 3),
  index   = c(seq(40, 65, length.out = length(dates)) + rnorm(length(dates), 0, 2),
              seq(55, 35, length.out = length(dates)) + rnorm(length(dates), 0, 2),
              seq(45, 70, length.out = length(dates)) + rnorm(length(dates), 0, 3)),
  n       = 1000
)
result <- extract(dat, n_col = "n", smoothing = FALSE)
print(result)

}
\references{
Stimson, J. A. (1991). \emph{Public Opinion in America: Moods, Cycles, and
Swings}. Westview Press.

Stimson, J. A. (1999). \emph{Public Opinion in America}, 2nd ed.
Westview Press.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_document.R
\docType{data}
\name{jennings}
\alias{jennings}
\title{Jennings Government Trust Data}
\format{
A data frame with 4 variables and `r nrow(jennings)` rows:
\describe{
  \item{variable}{Character string identifying the survey question or series.}
  \item{date}{Date the survey was fielded.}
  \item{value}{percentage of people indicating distrust in the government.}
  \item{n}{Sample size for the survey wave.}
}
}
\source{
Jennings, W. N. Clarke, J. Moss and G. Stoker  (2017). "The Decline in Diffuse Support for National Politics: The Long View on Political Discontent in Britain" In *Public Opinion Quarterly*,
 81(3), 748-758. \doi{10.1093/poq/nfx020}
}
\description{
A dataset of survey marginals from the British Social Attitudes (BSA) survey,
measuring public trust in government. These marginals are commonly used as
input to the Dyad Ratios Algorithm for constructing latent time series.  We 
replaced missing sample sizes with a value of 850, which is roughly the minimum
sample size observed in the data.
}
\examples{
data(jennings)
head(jennings)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_mood.R
\name{get_mood}
\alias{get_mood}
\title{Extract mood estimates as a data frame}
\usage{
get_mood(obj, ...)
}
\arguments{
\item{obj}{An object of class \code{"extract"} returned by
\code{\link{extract}}.}

\item{...}{Ignored; included for potential future use.}
}
\value{
A \code{data.frame} with one row per time period.  Columns
  \code{period}, \code{year}, \code{month}, and \code{quarter} are
  inherited from the period table inside \code{obj}.  When a single
  dimension was estimated an additional column \code{mood} is appended.
  When two dimensions were estimated, columns \code{mood_dim1} and
  \code{mood_dim2} are appended instead.
}
\description{
Combines the period descriptor table from an \code{\link{extract}} result
with the estimated mood trajectory, returning a plain \code{data.frame}
that is convenient for further analysis or export.
}
\examples{
set.seed(1)
dat <- data.frame(
  varname = rep(c("a", "b", "c"), each = 20),
  date    = rep(seq(as.Date("1980-01-01"), by = "year", length.out = 20), 3),
  index   = 50 + rnorm(60, 0, 5),
  n       = 1000L
)
res <- extract(dat, n_col = "n", smoothing = FALSE)
mood_df <- get_mood(res)
head(mood_df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{boot_dr}
\alias{boot_dr}
\title{Bootstrap the Dyad Ratios estimate}
\usage{
boot_dr(
  obj,
  data,
  R = 200L,
  level = 0.95,
  pw = FALSE,
  seed = NULL,
  parallel = FALSE
)
}
\arguments{
\item{obj}{An object of class \code{"extract"} returned by
\code{\link{extract}}.  The stored \code{call} is used to replay the
estimation identically on each bootstrap draw.}

\item{data}{The original data frame that was passed to \code{\link{extract}}
to produce \code{obj}.  The index column is perturbed for each replication;
all other columns are used as-is.}

\item{R}{Integer. Number of bootstrap replications. Default \code{200}.}

\item{level}{Numeric in (0, 1). Confidence level for the interval.
Default \code{0.95}.}

\item{pw}{Logical. Whether to calculate pairwise differences between times}

\item{seed}{Integer or \code{NULL}. Passed to \code{set.seed} before
the bootstrap loop. Default \code{NULL}.}

\item{parallel}{Logical. Parallelise the outer loop using \pkg{parallel} if
available? Useful for large \code{R}. Default \code{FALSE}.}
}
\value{
A list of class \code{"boot_dr"} with two or four components
  depending on whether the original \code{obj} used one or two dimensions:
  \describe{
    \item{\code{estimates}}{A \code{data.frame} with one row per time period
      and columns \code{period}, \code{year}, \code{month}, \code{quarter},
      \code{mood} (original point estimate), \code{lower}, and \code{upper}
      (confidence bounds at \code{(1-level)/2} and \code{1-(1-level)/2}).
      When \code{n_dim = 2}, three additional columns are appended:
      \code{mood_dim2}, \code{lower_dim2}, and \code{upper_dim2}.}
    \item{\code{samples}}{An \code{n_periods} × \code{R} matrix of raw
      bootstrap dimension-1 trajectories.}
    \item{\code{samples_dim2}}{(\code{n_dim = 2} only) An \code{n_periods}
      × \code{R} matrix of raw bootstrap dimension-2 trajectories.}
  }
  The list also carries attributes \code{R} (number of successful
  replications), \code{level}, \code{agg_interval}, and \code{n_dim}.
}
\description{
The bootstrapping algorithm was added to the package and code by Dave Armstrong. 
It is worth noting that this was not something Stimson had originally implemented
nor even something that he necessarily even endorsed.  While bootstrapping is a common
method for estimating uncertainty in complex estimators, like this one, use these
results at your own risk.
}
\details{
Generates a sampling distribution around the latent mood trajectory by
repeatedly drawing synthetic survey marginals from a binomial model and
re-running \code{\link{extract}}.  The original (unperturbed) estimate is
used as the point estimate; the bootstrap draws characterise uncertainty
around it.  

All model parameters (aggregation interval, column names, smoothing, etc.)
are taken directly from the stored call inside \code{obj}, so there is no
risk of the bootstrap replications being run with different settings than
the original.


Each replication draws \eqn{y_i \sim \text{Binomial}(n_i, p_i)} where
\eqn{p_i} is the observed proportion and \eqn{n_i} is the sample size, then
replaces the index column with \eqn{y_i / n_i} (rescaled to the same 0–100
vs 0–1 convention as the original) and re-runs \code{\link{extract}} using
the exact call stored in \code{obj}.  Replications that error (e.g. due to
a degenerate draw) are silently discarded; \code{attr(result, "R")} reports
how many succeeded.
}
\examples{
# Build a small synthetic dataset: 4 items measured annually over 20 years
set.seed(42)
n_years <- 20
years   <- seq(1980, length.out = n_years)
items   <- c("item_a", "item_b", "item_c", "item_d")

dat <- do.call(rbind, lapply(items, function(item) {
  data.frame(
    varname = item,
    date    = as.Date(paste0(years, "-07-01")),
    index   = 50 + cumsum(rnorm(n_years, 0, 1.5)) + rnorm(n_years, 0, 2),
    n       = sample(800:1200, n_years, replace = TRUE)
  )
}))

# Run the original estimate first
res <- extract(dat, n_col = "n", smoothing = FALSE)

# Bootstrap with 100 replications (use more in practice)
boot <- boot_dr(res, dat, R = 100, seed = 1)

# estimates is the summary data frame
head(boot$estimates)

# mood is the original point estimate; lower/upper are the 95\% CI
boot$estimates[, c("year", "mood", "lower", "upper")]

# samples is the raw n_periods x R matrix of bootstrap trajectories
dim(boot$samples)

# Plot the trajectory with uncertainty ribbon
plot(boot)

}
\seealso{
\code{\link{extract}}, \code{\link{plot.boot_dr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{plot.boot_dr}
\alias{plot.boot_dr}
\title{Plot a boot_dr object}
\usage{
\method{plot}{boot_dr}(x, dim = 1L, title = "Estimated Public Mood", ylab = "Mood", ...)
}
\arguments{
\item{x}{A \code{boot_dr} object returned by \code{\link{boot_dr}}.}

\item{dim}{Integer.  Which dimension to plot (\code{1} or \code{2}).
Dimension 2 is only available when the original \code{\link{extract}}
call used \code{n_dim = 2}.  Default \code{1}.}

\item{title}{Character. Plot title.}

\item{ylab}{Character. Y-axis label.}

\item{...}{Ignored.}
}
\value{
Invisibly, the ggplot2 object or \code{NULL} (base R).
}
\description{
Draws a ribbon plot of the bootstrap confidence interval around the
original mood estimate.  Requires \pkg{ggplot2}; falls back to base R if
unavailable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{plot.extract}
\alias{plot.extract}
\title{Plot estimated public mood}
\usage{
\method{plot}{extract}(x, dim = 1L, title = "Estimated Public Mood", ylab = "Mood", ...)
}
\arguments{
\item{x}{A \code{extract} object.}

\item{dim}{Integer.  Which dimension to plot (1 or 2).  Default \code{1}.}

\item{title}{Character.  Plot title.  Default \code{"Estimated Public Mood"}.}

\item{ylab}{Character.  Y-axis label.  Default \code{"Mood"}.}

\item{...}{Additional arguments passed to the plotting function.}
}
\value{
Invisibly, the plot object (ggplot2) or \code{NULL} (base R).
}
\description{
Produces a time-series plot of the estimated latent mood.  Requires
\pkg{ggplot2}.  If \pkg{ggplot2} is not installed, falls back to base R
graphics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dyadratios-package.R
\docType{package}
\name{dyadratios-package}
\alias{DyadRatios}
\alias{DyadRatios-package}
\alias{dyadratios-package}
\title{dyadratios: Dyad Ratios Algorithm for Latent Public Opinion Estimation}
\description{
Implements the Dyad Ratios algorithm (Stimson 1991) for estimating latent
public mood from a collection of time-series survey marginals.  The
computationally intensive estimation loop is written in C++ (via Rcpp) and
is a faithful translation of James Stimson's original 'MCalc' program.  The R
layer handles data ingestion, temporal aggregation, result formatting, and
visualisation.
}
\section{Main functions}{

\describe{
  \item{\code{\link{extract}}}{Run the algorithm on a data frame.}
}
}

\section{Input data format}{

The primary input is a data frame where each row is one survey marginal:
\itemize{
  \item A column identifying the opinion \strong{variable} (issue series).
  \item A \strong{date} column (any format coercible by \code{as.Date}).
  \item An \strong{index} column: the survey proportion or percentage.
  \item An optional \strong{n} column: number of respondents (used as
        weight; defaults to 1000 if omitted).
}
}

\references{
Stimson, J. A. (1991). \emph{Public Opinion in America: Moods, Cycles, and
Swings}. Boulder, CO: Westview Press.

Stimson, J. A. (1999). \emph{Public Opinion in America}, 2nd ed. Westview.
}
\author{
\strong{Maintainer}: Dave Armstrong \email{davearmstrong.ps@gmail.com}

Authors:
\itemize{
  \item James Stimson (Original C++ implementation)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.extract}
\alias{summary.extract}
\title{Summary method for extract objects}
\usage{
\method{summary}{extract}(object, ...)
}
\arguments{
\item{object}{A \code{extract} object.}

\item{...}{Ignored.}
}
\value{
Invisibly returns \code{object}.
}
\description{
Prints a detailed report similar to the log file produced by the original
'MCalc' program, including the iteration history, variable loadings, and
variance accounting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.extract}
\alias{print.extract}
\title{Print method for extract objects}
\usage{
\method{print}{extract}(x, ...)
}
\arguments{
\item{x}{A \code{extract} object returned by \code{\link{extract}}.}

\item{...}{Ignored.}
}
\description{
Print method for extract objects
}
